@@ -7,11 +7,11 @@ from contextlib import contextmanager
 from dataclasses import dataclass
 from decimal import ROUND_HALF_UP
 from functools import lru_cache, partial
-from io import BytesIO
+from io import BytesIO, IOBase
 from itertools import islice
 from math import ceil, sqrt
 from pathlib import Path
-from subprocess import PIPE, run
+from subprocess import CalledProcessError, PIPE, run
 from typing import (
     Any,
     Callable,
@@ -21,16 +21,24 @@ from typing import (
     Mapping,
     NamedTuple,
     Optional,
-    Sequence,
     Tuple,
     Union,
 )
 
 import numpy as np
+import torch
 from tqdm.auto import tqdm
 
-from lhotse.augmentation import AudioTransform, Resample, Speed, Tempo, Volume
+from lhotse.augmentation import (
+    AudioTransform,
+    Resample,
+    ReverbWithImpulseResponse,
+    Speed,
+    Tempo,
+    Volume,
+)
 from lhotse.caching import dynamic_lru_cache
+from lhotse.lazy import AlgorithmMixin
 from lhotse.serialization import Serializable
 from lhotse.utils import (
     Decibels,
@@ -47,12 +55,60 @@ from lhotse.utils import (
     index_by_id_and_check,
     perturb_num_samples,
     rich_exception_info,
+    split_manifest_lazy,
     split_sequence,
 )
 
 Channels = Union[int, List[int]]
 
 
+_DEFAULT_LHOTSE_AUDIO_DURATION_MISMATCH_TOLERANCE: Seconds = 0.025
+LHOTSE_AUDIO_DURATION_MISMATCH_TOLERANCE: Seconds = (
+    _DEFAULT_LHOTSE_AUDIO_DURATION_MISMATCH_TOLERANCE
+)
+
+
+def set_audio_duration_mismatch_tolerance(delta: Seconds) -> None:
+    """
+    Override Lhotse's global threshold for allowed audio duration mismatch between the
+    manifest and the actual data.
+
+    Some scenarios when a mismatch can happen:
+
+        - the :class:`.Recording` manifest duration is rounded off too much
+            (i.e., bad user input, but too inconvenient to go back and fix the manifests)
+
+        - data augmentation changes the number of samples a bit in a difficult to predict way
+
+    When there is a mismatch, Lhotse will either trim or replicate the diff to match
+    the value found in the :class:`.Recording` manifest.
+
+    .. note:: We don't recommend setting this lower than the default value, as it could
+        break some data augmentation transforms.
+
+    Example::
+
+        >>> import lhotse
+        >>> lhotse.set_audio_duration_mismatch_tolerance(0.01)  # 10ms tolerance
+
+    :param delta: New tolerance in seconds.
+    """
+    global LHOTSE_AUDIO_DURATION_MISMATCH_TOLERANCE
+    logging.info(
+        "The user overrided tolerance for audio duration mismatch "
+        "between the values in the manifest and the actual data. "
+        f"Old threshold: {LHOTSE_AUDIO_DURATION_MISMATCH_TOLERANCE}s. "
+        f"New threshold: {delta}s."
+    )
+    if delta < _DEFAULT_LHOTSE_AUDIO_DURATION_MISMATCH_TOLERANCE:
+        warnings.warn(
+            "The audio duration mismatch tolerance has been set to a value lower than "
+            f"default ({_DEFAULT_LHOTSE_AUDIO_DURATION_MISMATCH_TOLERANCE}s). "
+            f"We don't recommend this as it might break some data augmentation transforms."
+        )
+    LHOTSE_AUDIO_DURATION_MISMATCH_TOLERANCE = delta
+
+
 # TODO: document the dataclasses like this:
 # https://stackoverflow.com/a/3051356/5285891
 
@@ -65,11 +121,12 @@ class AudioSource:
     - 'file' (formats supported by soundfile, possibly multi-channel)
     - 'command' [unix pipe] (must be WAVE, possibly multi-channel)
     - 'url' (any URL type that is supported by "smart_open" library, e.g. http/https/s3/gcp/azure/etc.)
+    - 'memory' (any format, read from a binary string attached to 'source' member of AudioSource)
     """
 
     type: str
     channels: List[int]
-    source: str
+    source: Union[str, bytes]
 
     def load_audio(
         self,
@@ -84,12 +141,12 @@ class AudioSource:
         (n_channels, n_samples) for multi-channel.
 
         Note: The elements in the returned array are in the range [-1.0, 1.0]
-        and are of dtype `np.floatt32`.
+        and are of dtype `np.float32`.
 
         :param force_opus_sampling_rate: This parameter is only used when we detect an OPUS file.
             It will tell ffmpeg to resample OPUS to this sampling rate.
         """
-        assert self.type in ("file", "command", "url")
+        assert self.type in ("file", "command", "url", "memory")
 
         # TODO: refactor when another source type is added
         source = self.source
@@ -125,6 +182,16 @@ class AudioSource:
                     source, offset=offset, duration=duration
                 )
 
+        elif self.type == "memory":
+            assert isinstance(self.source, bytes), (
+                "Corrupted manifest: specified AudioSource type is 'memory', "
+                "but 'self.source' attribute is not of type 'bytes'."
+            )
+            source = BytesIO(self.source)
+            samples, sampling_rate = read_audio(
+                source, offset=offset, duration=duration
+            )
+
         else:  # self.type == 'file'
             samples, sampling_rate = read_audio(
                 source,
@@ -140,9 +207,9 @@ class AudioSource:
             )
             available_duration = num_samples / sampling_rate
             if (
-                available_duration < duration - 1e-3
+                available_duration < duration - LHOTSE_AUDIO_DURATION_MISMATCH_TOLERANCE
             ):  # set the allowance as 1ms to avoid float error
-                raise ValueError(
+                raise DurationMismatchError(
                     f"Requested more audio ({duration}s) than available ({available_duration}s)"
                 )
 
@@ -160,6 +227,12 @@ class AudioSource:
     def from_dict(data) -> "AudioSource":
         return AudioSource(**data)
 
+    def __repr__(self):
+        return (
+            f"AudioSource(type='{self.type}', channels={self.channels}, "
+            f"source='{self.source if isinstance(self.source, str) else '<binary-data>'}')"
+        )
+
 
 @dataclass
 class Recording:
@@ -242,9 +315,10 @@ class Recording:
     @staticmethod
     def from_file(
         path: Pathlike,
-        recording_id: Optional[str] = None,
+        recording_id: Optional[Union[str, Callable[[Path], str]]] = None,
         relative_path_depth: Optional[int] = None,
         force_opus_sampling_rate: Optional[int] = None,
+        force_read_audio: bool = False,
     ) -> "Recording":
         """
         Read an audio file's header and create the corresponding ``Recording``.
@@ -257,18 +331,32 @@ class Recording:
 
         :param path: Path to an audio file supported by libsoundfile (pysoundfile).
         :param recording_id: recording id, when not specified ream the filename's stem ("x.wav" -> "x").
+            It can be specified as a string or a function that takes the recording path and returns a string.
         :param relative_path_depth: optional int specifying how many last parts of the file path
             should be retained in the ``AudioSource``. By default writes the path as is.
         :param force_opus_sampling_rate: when specified, this value will be used as the sampling rate
             instead of the one we read from the manifest. This is useful for OPUS files that always
             have 48kHz rate and need to be resampled to the real one -- we will perform that operation
             "under-the-hood". For non-OPUS files this input is undefined.
+        :param force_read_audio: Set it to ``True`` for audio files that do not have any metadata
+            in their headers (e.g., "The People's Speech" FLAC files).
         :return: a new ``Recording`` instance pointing to the audio file.
         """
         path = Path(path)
-        audio_info = info(path, force_opus_sampling_rate=force_opus_sampling_rate)
+        recording_id = (
+            path.stem
+            if recording_id is None
+            else recording_id(path)
+            if callable(recording_id)
+            else recording_id
+        )
+        audio_info = info(
+            path,
+            force_opus_sampling_rate=force_opus_sampling_rate,
+            force_read_audio=force_read_audio,
+        )
         return Recording(
-            id=recording_id if recording_id is not None else path.stem,
+            id=recording_id,
             sampling_rate=audio_info.samplerate,
             num_samples=audio_info.frames,
             duration=audio_info.duration,
@@ -285,9 +373,147 @@ class Recording:
             ],
         )
 
+    @staticmethod
+    def from_bytes(
+        data: bytes,
+        recording_id: str,
+    ) -> "Recording":
+        """
+        Like :meth:`.Recording.from_file`, but creates a manifest for a byte string with
+        raw encoded audio data. This data is first decoded to obtain info such as the
+        sampling rate, number of channels, etc. Then, the binary data is attached to the
+        manifest. Calling :meth:`.Recording.load_audio` does not perform any I/O and
+        instead decodes the byte string contents in memory.
+
+        .. note:: Intended use of this method is for packing Recordings into archives
+            where metadata and data should be available together
+            (e.g., in WebDataset style tarballs).
+
+        .. caution:: Manifest created with this method cannot be stored as JSON
+            because JSON doesn't allow serializing binary data.
+
+        :param data: bytes, byte string containing encoded audio contents.
+        :param recording_id: recording id, unique string identifier.
+        :return: a new ``Recording`` instance that owns the byte string data.
+        """
+        stream = BytesIO(data)
+        audio_info = torchaudio_info(stream)
+        return Recording(
+            id=recording_id,
+            sampling_rate=audio_info.samplerate,
+            num_samples=audio_info.frames,
+            duration=audio_info.duration,
+            sources=[
+                AudioSource(
+                    type="memory",
+                    channels=list(range(audio_info.channels)),
+                    source=data,
+                )
+            ],
+        )
+
+    def move_to_memory(
+        self,
+        channels: Optional[Channels] = None,
+        offset: Seconds = None,
+        duration: Optional[Seconds] = None,
+        format: Optional[str] = None,
+    ) -> "Recording":
+        """
+        Read audio data and return a copy of the manifest with binary data attached.
+        Calling :meth:`.Recording.load_audio` on that copy will not trigger I/O.
+
+        If all arguments are left as defaults, we won't decode the audio and attach
+        the bytes we read from disk/other source as-is.
+        If ``channels``, ``duration``, or ``offset`` are specified, we'll decode the
+        audio and re-encode it into ``format`` before attaching.
+        The default format is FLAC, other formats compatible with torchaudio.save are
+        also accepted.
+        """
+
+        if all(src.type == "memory" for src in self.sources):
+            return self  # nothing to do
+
+        # Case #1: no opts specified, read audio without decoding and move it in memory.
+        if all(opt is None for opt in (channels, offset, duration)):
+            memory_sources = [
+                AudioSource(
+                    type="memory",
+                    channels=old_source.channels,
+                    source=open(old_source.source, "rb").read(),
+                )
+                for old_source in self.sources
+            ]
+            return fastcopy(self, sources=memory_sources)
+
+        # Case #2: user specified some subset of the recording, decode audio,
+        #          subset it, and encode it again but save in memory.
+        import torchaudio
+
+        audio = self.load_audio(
+            channels=channels, offset=ifnone(offset, 0), duration=duration
+        )
+        stream = BytesIO()
+        torchaudio.save(
+            stream, torch.from_numpy(audio), self.sampling_rate, format=format
+        )
+        channels = (ifnone(channels, self.channel_ids),)
+        if isinstance(channels, int):
+            channels = [channels]
+        return Recording(
+            id=self.id,
+            sources=[
+                AudioSource(
+                    type="memory",
+                    channels=channels,
+                    source=stream.getvalue(),
+                )
+            ],
+            sampling_rate=self.sampling_rate,
+            num_samples=audio.shape[1],
+            duration=ifnone(duration, self.duration),
+        )
+
     def to_dict(self) -> dict:
         return asdict_nonull(self)
 
+    def to_cut(self):
+        """
+        Create a Cut out of this recording.
+
+        For single-channel recordings, we return a :class:`MonoCut`.
+        For multi-channel recordings, we return a :class:`MixedCut` with as
+        many tracks as the number of channels.
+        The implementation of the multi-channel case may change in the future...
+        """
+        from lhotse.cut import MonoCut, MixedCut, MixTrack
+
+        if self.num_channels == 1:
+            return MonoCut(
+                id=self.id,
+                start=0.0,
+                duration=self.duration,
+                channel=self.channel_ids[0],
+                recording=self,
+            )
+        else:
+            # TODO: if we ever have "MultiCut" we may replace this implementation
+            return MixedCut(
+                id=self.id,
+                tracks=[
+                    MixTrack(
+                        cut=MonoCut(
+                            id=f"{self.id}_ch{cidx}",
+                            start=0.0,
+                            duration=self.duration,
+                            channel=cidx,
+                            recording=self,
+                        )
+                    )
+                    for cidx in self.channel_ids
+                ],
+            )
+
     @property
     def num_channels(self):
         return sum(len(source.channels) for source in self.sources)
@@ -313,6 +539,12 @@ class Recording:
         :param duration: seconds, indicates the total audio time to read (starting from ``offset``).
         :return: a numpy array of audio samples with shape ``(num_channels, num_samples)``.
         """
+
+        assert offset <= self.duration, (
+            f"Cannot load audio because the Recording's duration {self.duration}s "
+            f"is smaller than the requested offset {offset}s."
+        )
+
         if channels is None:
             channels = SetContainingAnything()
         else:
@@ -450,19 +682,70 @@ class Recording:
             transforms=transforms,
         )
 
+    def reverb_rir(
+        self,
+        rir_recording: "Recording",
+        normalize_output: bool = True,
+        early_only: bool = False,
+        affix_id: bool = True,
+        rir_channels: Optional[List[int]] = None,
+    ) -> "Recording":
+        """
+        Return a new ``Recording`` that will lazily apply reverberation based on provided
+        impulse response while loading audio.
+
+        :param rir_recording: The impulse response to be used.
+        :param normalize_output: When true, output will be normalized to have energy as input.
+        :param early_only: When true, only the early reflections (first 50 ms) will be used.
+        :param affix_id: When true, we will modify the ``Recording.id`` field
+            by affixing it with "_rvb".
+        :param rir_channels: The channels of the impulse response to be used (in case of multi-channel
+            impulse responses). By default, only the first channel is used.
+        :return: the perturbed ``Recording``.
+        """
+        transforms = self.transforms.copy() if self.transforms is not None else []
+        transforms.append(
+            ReverbWithImpulseResponse(
+                rir_recording,
+                normalize_output=normalize_output,
+                early_only=early_only,
+                rir_channels=rir_channels if rir_channels is not None else [0],
+            ).to_dict()
+        )
+        return fastcopy(
+            self,
+            id=f"{self.id}_rvb" if affix_id else self.id,
+            transforms=transforms,
+        )
+
     def resample(self, sampling_rate: int) -> "Recording":
         """
         Return a new ``Recording`` that will be lazily resampled while loading audio.
         :param sampling_rate: The new sampling rate.
         :return: A resampled ``Recording``.
         """
+        if sampling_rate == self.sampling_rate:
+            return fastcopy(self)
+
         transforms = self.transforms.copy() if self.transforms is not None else []
-        transforms.append(
-            Resample(
-                source_sampling_rate=self.sampling_rate,
-                target_sampling_rate=sampling_rate,
-            ).to_dict()
-        )
+
+        if not any(
+            isinstance(s.source, str) and s.source.endswith(".opus")
+            for s in self.sources
+        ):
+            # OPUS is a special case for resampling.
+            # Normally, we use Torchaudio SoX bindings for resampling,
+            # but in case of OPUS we ask FFMPEG to resample it during
+            # decoding as its faster.
+            # Because of that, we have to skip adding a transform
+            # for OPUS files and only update the metadata in the manifest.
+            transforms.append(
+                Resample(
+                    source_sampling_rate=self.sampling_rate,
+                    target_sampling_rate=sampling_rate,
+                ).to_dict()
+            )
+
         new_num_samples = compute_num_samples(
             self.duration, sampling_rate, rounding=ROUND_HALF_UP
         )
@@ -486,7 +769,7 @@ class Recording:
         )
 
 
-class RecordingSet(Serializable, Sequence[Recording]):
+class RecordingSet(Serializable, AlgorithmMixin):
     """
     :class:`~lhotse.audio.RecordingSet` represents a collection of recordings, indexed by recording IDs.
     It does not contain any annotation such as the transcript or the speaker identity --
@@ -544,6 +827,7 @@ class RecordingSet(Serializable, Sequence[Recording]):
 
             >>> recs_sp = recs.perturb_speed(factor=1.1)
             >>> recs_vp = recs.perturb_volume(factor=2.)
+            >>> recs_rvb = recs.reverb_rir(rir_recs)
             >>> recs_24k = recs.resample(24000)
     """
 
@@ -554,13 +838,9 @@ class RecordingSet(Serializable, Sequence[Recording]):
         return self.recordings == other.recordings
 
     @property
-    def is_lazy(self) -> bool:
-        """
-        Indicates whether this manifest was opened in lazy (read-on-the-fly) mode or not.
-        """
-        from lhotse.serialization import LazyJsonlIterator
-
-        return isinstance(self.recordings, LazyJsonlIterator)
+    def data(self) -> Union[Dict[str, Recording], Iterable[Recording]]:
+        """Alias property for ``self.recordings``"""
+        return self.recordings
 
     @property
     def ids(self) -> Iterable[str]:
@@ -570,12 +850,15 @@ class RecordingSet(Serializable, Sequence[Recording]):
     def from_recordings(recordings: Iterable[Recording]) -> "RecordingSet":
         return RecordingSet(recordings=index_by_id_and_check(recordings))
 
+    from_items = from_recordings
+
     @staticmethod
     def from_dir(
         path: Pathlike,
         pattern: str,
         num_jobs: int = 1,
         force_opus_sampling_rate: Optional[int] = None,
+        recording_id: Optional[Callable[[Path], str]] = None,
     ):
         """
         Recursively scan a directory ``path`` for audio files that match the given ``pattern`` and create
@@ -595,22 +878,29 @@ class RecordingSet(Serializable, Sequence[Recording]):
             instead of the one we read from the manifest. This is useful for OPUS files that always
             have 48kHz rate and need to be resampled to the real one -- we will perform that operation
             "under-the-hood". For non-OPUS files this input does nothing.
+        :param recording_id: A function which takes the audio file path and returns the recording ID. If not
+            specified, the filename will be used as the recording ID.
         :return: a new ``Recording`` instance pointing to the audio file.
         """
         msg = f"Scanning audio files ({pattern})"
-        fn = Recording.from_file
-        if force_opus_sampling_rate is not None:
-            fn = partial(
-                Recording.from_file, force_opus_sampling_rate=force_opus_sampling_rate
-            )
+
+        file_read_worker = partial(
+            Recording.from_file,
+            force_opus_sampling_rate=force_opus_sampling_rate,
+            recording_id=recording_id,
+        )
+
         if num_jobs == 1:
             # Avoid spawning process for one job.
             return RecordingSet.from_recordings(
-                tqdm(map(fn, Path(path).rglob(pattern)), desc=msg)
+                tqdm(map(file_read_worker, Path(path).rglob(pattern)), desc=msg)
             )
         with ProcessPoolExecutor(num_jobs) as ex:
             return RecordingSet.from_recordings(
-                tqdm(ex.map(fn, Path(path).rglob(pattern)), desc=msg)
+                tqdm(
+                    ex.map(file_read_worker, Path(path).rglob(pattern)),
+                    desc=msg,
+                )
             )
 
     @staticmethod
@@ -622,28 +912,6 @@ class RecordingSet(Serializable, Sequence[Recording]):
     def to_dicts(self) -> Iterable[dict]:
         return (r.to_dict() for r in self)
 
-    def filter(self, predicate: Callable[[Recording], bool]) -> "RecordingSet":
-        """
-        Return a new RecordingSet with the Recordings that satisfy the `predicate`.
-
-        :param predicate: a function that takes a recording as an argument and returns bool.
-        :return: a filtered RecordingSet.
-        """
-        return RecordingSet.from_recordings(rec for rec in self if predicate(rec))
-
-    def shuffle(self, rng: Optional[random.Random] = None) -> "RecordingSet":
-        """
-        Shuffle the recording IDs in the current :class:`.RecordingSet` and return a shuffled copy of self.
-
-        :param rng: an optional instance of ``random.Random`` for precise control of randomness.
-        :return: a shuffled copy of self.
-        """
-        if rng is None:
-            rng = random
-        ids = list(self.ids)
-        rng.shuffle(ids)
-        return RecordingSet(recordings={rid: self[rid] for rid in ids})
-
     def split(
         self, num_splits: int, shuffle: bool = False, drop_last: bool = False
     ) -> List["RecordingSet"]:
@@ -665,6 +933,24 @@ class RecordingSet(Serializable, Sequence[Recording]):
             )
         ]
 
+    def split_lazy(self, output_dir: Pathlike, chunk_size: int) -> List["RecordingSet"]:
+        """
+        Splits a manifest (either lazily or eagerly opened) into chunks, each
+        with ``chunk_size`` items (except for the last one, typically).
+
+        In order to be memory efficient, this implementation saves each chunk
+        to disk in a ``.jsonl.gz`` format as the input manifest is sampled.
+
+        .. note:: For lowest memory usage, use ``load_manifest_lazy`` to open the
+            input manifest for this method.
+
+        :param output_dir: directory where the split manifests are saved.
+            Each manifest is saved at: ``{output_dir}/{split_idx}.jsonl.gz``
+        :param chunk_size: the number of items in each chunk.
+        :return: a list of lazily opened chunk manifests.
+        """
+        return split_manifest_lazy(self, output_dir=output_dir, chunk_size=chunk_size)
+
     def subset(
         self, first: Optional[int] = None, last: Optional[int] = None
     ) -> "RecordingSet":
@@ -682,13 +968,12 @@ class RecordingSet(Serializable, Sequence[Recording]):
 
         if first is not None:
             assert first > 0
-            if first > len(self):
+            out = RecordingSet.from_items(islice(self, first))
+            if len(out) < first:
                 logging.warning(
-                    f"RecordingSet has only {len(self)} items but first {first} required; "
-                    f"not doing anything."
+                    f"RecordingSet has only {len(out)} items but first {first} were requested."
                 )
-                return self
-            return RecordingSet.from_recordings(islice(self, first))
+            return out
 
         if last is not None:
             assert last > 0
@@ -771,6 +1056,39 @@ class RecordingSet(Serializable, Sequence[Recording]):
             r.perturb_volume(factor=factor, affix_id=affix_id) for r in self
         )
 
+    def reverb_rir(
+        self,
+        rir_recordings: "RecordingSet",
+        normalize_output: bool = True,
+        early_only: bool = False,
+        affix_id: bool = True,
+        rir_channels: List[int] = [0],
+    ) -> "RecordingSet":
+        """
+        Return a new ``RecordingSet`` that will lazily apply reverberation based on provided
+        impulse responses while loading audio.
+
+        :param rir_recordings: The impulse responses to be used.
+        :param normalize_output: When true, output will be normalized to have energy as input.
+        :param early_only: When true, only the early reflections (first 50 ms) will be used.
+        :param affix_id: When true, we will modify the ``Recording.id`` field
+            by affixing it with "_rvb".
+        :param rir_channels: The channels to be used for the RIRs (if multi-channel). Uses first
+            channel by default.
+        :return: a ``RecordingSet`` containing the perturbed ``Recording`` objects.
+        """
+        rir_recordings = list(rir_recordings)
+        return RecordingSet.from_recordings(
+            r.reverb_rir(
+                rir_recording=random.choice(rir_recordings),
+                normalize_output=normalize_output,
+                early_only=early_only,
+                affix_id=affix_id,
+                rir_channels=rir_channels,
+            )
+            for r in self
+        )
+
     def resample(self, sampling_rate: int) -> "RecordingSet":
         """
         Apply resampling to all recordings in the ``RecordingSet`` and return a new ``RecordingSet``.
@@ -804,9 +1122,6 @@ class RecordingSet(Serializable, Sequence[Recording]):
     def __len__(self) -> int:
         return len(self.recordings)
 
-    def __add__(self, other: "RecordingSet") -> "RecordingSet":
-        return RecordingSet(recordings={**self.recordings, **other.recordings})
-
 
 class AudioMixer:
     """
@@ -822,17 +1137,37 @@ class AudioMixer:
     The SNR is relative to the energy of the signal used to initialize the ``AudioMixer``.
     """
 
-    def __init__(self, base_audio: np.ndarray, sampling_rate: int):
+    def __init__(
+        self,
+        base_audio: np.ndarray,
+        sampling_rate: int,
+        reference_energy: Optional[float] = None,
+    ):
         """
+        AudioMixer's constructor.
+
         :param base_audio: A numpy array with the audio samples for the base signal
             (all the other signals will be mixed to it).
         :param sampling_rate: Sampling rate of the audio.
+        :param reference_energy: Optionally pass a reference energy value to compute SNRs against.
+            This might be required when ``base_audio`` corresponds to zero-padding.
         """
         self.tracks = [base_audio]
         self.sampling_rate = sampling_rate
-        self.reference_energy = audio_energy(base_audio)
         self.dtype = self.tracks[0].dtype
 
+        # Keep a pre-computed energy value of the audio that we initialize the Mixer with;
+        # it is required to compute gain ratios that satisfy SNR during the mix.
+        if reference_energy is None:
+            self.reference_energy = audio_energy(base_audio)
+        else:
+            self.reference_energy = reference_energy
+
+        if self.reference_energy <= 0.0:
+            raise NonPositiveEnergyError(
+                f"To perform mix, energy must be non-zero and non-negative (got {self.reference_energy})"
+            )
+
     @property
     def unmixed_audio(self) -> np.ndarray:
         """
@@ -864,6 +1199,9 @@ class AudioMixer:
         the start with low energy values.
         :return:
         """
+        if len(audio) == 0:
+            return  # do nothing for empty arrays
+
         assert audio.shape[0] == 1  # TODO: support multi-channels
         assert offset >= 0.0, "Negative offset in mixing is not supported."
 
@@ -929,7 +1267,7 @@ class AudioMixer:
 
 
 def audio_energy(audio: np.ndarray) -> float:
-    return float(np.average(audio ** 2))
+    return float(np.average(audio**2))
 
 
 FileObject = Any  # Alias for file-like objects
@@ -944,26 +1282,31 @@ def read_audio(
 ) -> Tuple[np.ndarray, int]:
     # First handle special cases: OPUS and SPHERE (SPHERE may be encoded with shorten,
     #   which can only be decoded by binaries "shorten" and "sph2pipe").
-    if isinstance(path_or_fd, (str, Path)) and str(path_or_fd).lower().endswith(
-        ".opus"
-    ):
-        return read_opus(
-            path_or_fd,
-            offset=offset,
-            duration=duration,
-            force_opus_sampling_rate=force_opus_sampling_rate,
-        )
-    elif isinstance(path_or_fd, (str, Path)) and str(path_or_fd).lower().endswith(
-        ".sph"
-    ):
-        return read_sph(path_or_fd, offset=offset, duration=duration)
     try:
-        return torchaudio_load(path_or_fd, offset=offset, duration=duration)
-    except:
+        if isinstance(path_or_fd, (str, Path)) and str(path_or_fd).lower().endswith(
+            ".opus"
+        ):
+            return read_opus(
+                path_or_fd,
+                offset=offset,
+                duration=duration,
+                force_opus_sampling_rate=force_opus_sampling_rate,
+            )
+        elif isinstance(path_or_fd, (str, Path)) and str(path_or_fd).lower().endswith(
+            ".sph"
+        ):
+            return read_sph(path_or_fd, offset=offset, duration=duration)
         try:
-            return soundfile_load(path_or_fd, offset=offset, duration=duration)
+            return torchaudio_load(path_or_fd, offset=offset, duration=duration)
         except:
-            return audioread_load(path_or_fd, offset=offset, duration=duration)
+            try:
+                return soundfile_load(path_or_fd, offset=offset, duration=duration)
+            except:
+                return audioread_load(path_or_fd, offset=offset, duration=duration)
+    except Exception as e:
+        raise AudioLoadingError(
+            f"Reading audio from '{path_or_fd}' failed. Details: {type(e)}('{str(e)}')"
+        )
 
 
 class LibsndfileCompatibleAudioInfo(NamedTuple):
@@ -974,15 +1317,27 @@ class LibsndfileCompatibleAudioInfo(NamedTuple):
 
 
 def info(
-    path: Pathlike, force_opus_sampling_rate: Optional[int] = None
+    path: Pathlike,
+    force_opus_sampling_rate: Optional[int] = None,
+    force_read_audio: bool = False,
 ) -> LibsndfileCompatibleAudioInfo:
+
+    if force_read_audio:
+        # This is a reliable fallback for situations when the user knows that audio files do not
+        # have duration metadata in their headers.
+        # We will use "audioread" backend that spawns an ffmpeg process, reads the audio,
+        # and computes the duration.
+        return audioread_info(str(path))
+
     if path.suffix.lower() == ".opus":
         # We handle OPUS as a special case because we might need to force a certain sampling rate.
         return opus_info(path, force_opus_sampling_rate=force_opus_sampling_rate)
+
     elif path.suffix.lower() == ".sph":
         # We handle SPHERE as another special case because some old codecs (i.e. "shorten" codec)
         # can't be handled by neither pysoundfile nor pyaudioread.
         return sph_info(path)
+
     try:
         # Try to parse the file using torchaudio first.
         return torchaudio_info(path)
@@ -1017,22 +1372,8 @@ def torchaudio_info(path: Pathlike) -> LibsndfileCompatibleAudioInfo:
 def torchaudio_load(
     path_or_fd: Pathlike, offset: Seconds = 0, duration: Optional[Seconds] = None
 ) -> Tuple[np.ndarray, int]:
-    import torch
     import torchaudio
 
-    if not isinstance(path_or_fd, (str, Path)):
-        # Special case: we are likely dealing with a file descriptor (open file).
-        # If we run torchaudio.info() on it, it will consume some data,
-        # which will cause torchaudio.load() to fail.
-        # We expect offset and duration have default values, otherwise we fail.
-        assert offset == 0 and duration is None, (
-            "Lhotse doesn't support using torchaudio.load() "
-            "with open file objects when offset or duration "
-            "arguments are non-default."
-        )
-        audio, sampling_rate = torchaudio.load(path_or_fd)
-        return audio.numpy(), sampling_rate
-
     # Need to grab the "info" about sampling rate before reading to compute
     # the number of samples provided in offset / num_frames.
     audio_info = torchaudio_info(path_or_fd)
@@ -1042,39 +1383,15 @@ def torchaudio_load(
         frame_offset = compute_num_samples(offset, audio_info.samplerate)
     if duration is not None:
         num_frames = compute_num_samples(duration, audio_info.samplerate)
+    if isinstance(path_or_fd, IOBase):
+        # Set seek pointer to the beginning of the file as torchaudio.info
+        # might have left it at the end of the header
+        path_or_fd.seek(0)
     audio, sampling_rate = torchaudio.load(
         path_or_fd,
         frame_offset=frame_offset,
         num_frames=num_frames,
     )
-
-    # MP3 has weird behaviour sometimes: torchaudio.info() `num_frames` indicates
-    # a different number of samples than data shape from torchaudio.load().
-    # We'll truncate/zero-pad to ensure the shape is the same as from info,
-    # up to some threshold determined heuristically (25ms).
-    THRESHOLD: Seconds = 0.025
-    threshold_samples = THRESHOLD / sampling_rate
-    diff = audio.shape[1] - audio_info.frames
-    if diff < 0:
-        if abs(diff) <= threshold_samples:
-            audio = torch.nn.functional.pad(
-                audio, (0, abs(diff)), mode="constant", value=0
-            )
-        else:
-            raise ValueError(
-                f"Inconsistent audio data for '{path_or_fd}': torchaudio.info() declared "
-                f"{audio_info.frames} samples, but torchaudio.load() returned {audio.shape[1]} samples. "
-                f"Please report an issue in Lhotse or Torchaudio GitHub."
-            )
-    elif diff > 0:
-        if abs(diff) <= threshold_samples:
-            audio = audio[:, :-diff]
-        else:
-            raise ValueError(
-                f"Inconsistent audio data for '{path_or_fd}': torchaudio.info() declared "
-                f"{audio_info.frames} samples, but torchaudio.load() returned {audio.shape[1]} samples. "
-                f"Please report an issue in Lhotse or Torchaudio GitHub."
-            )
     return audio.numpy(), sampling_rate
 
 
@@ -1241,15 +1558,6 @@ def _buf_to_float(x, n_bytes=2, dtype=np.float32):
     return scale * np.frombuffer(x, fmt).astype(dtype)
 
 
-# This constant defines by how much our estimation can be mismatched with
-# the actual number of samples after applying audio augmentation.
-# Chains of augmentation effects (such as resampling, speed perturb) can cause
-# difficult to predict roundings and return a few samples more/less than we estimate.
-# The default tolerance is a quarter of a millisecond
-# (the actual number of samples is computed based on the sampling rate).
-AUGMENTATION_DURATION_TOLERANCE: Seconds = 0.00025
-
-
 def assert_and_maybe_fix_num_samples(
     audio: np.ndarray,
     offset: Seconds,
@@ -1268,10 +1576,11 @@ def assert_and_maybe_fix_num_samples(
     diff = expected_num_samples - audio.shape[1]
     if diff == 0:
         return audio  # this is normal condition
-    allowed_diff = int(ceil(AUGMENTATION_DURATION_TOLERANCE * recording.sampling_rate))
+    allowed_diff = int(
+        ceil(LHOTSE_AUDIO_DURATION_MISMATCH_TOLERANCE * recording.sampling_rate)
+    )
     if 0 < diff <= allowed_diff:
-        # note the extra colon in -1:, which preserves the shape
-        audio = np.append(audio, audio[:, -diff:], axis=1)
+        audio = np.pad(audio, ((0, 0), (0, diff)), mode="reflect")
         return audio
     elif -allowed_diff <= diff < 0:
         audio = audio[:, :diff]
@@ -1309,7 +1618,7 @@ def read_opus(
     """
     Reads OPUS files either using torchaudio or ffmpeg.
     Torchaudio is faster, but if unavailable for some reason,
-    we fallback to a slower ffmpeg-based implemention.
+    we fallback to a slower ffmpeg-based implementation.
 
     :return: a tuple of audio samples and the sampling rate.
     """
@@ -1374,7 +1683,7 @@ def read_opus_ffmpeg(
     :return: a tuple of audio samples and the sampling rate.
     """
     # Construct the ffmpeg command depending on the arguments passed.
-    cmd = f"ffmpeg"
+    cmd = "ffmpeg -threads 1"
     sampling_rate = 48000
     # Note: we have to add offset and duration options (-ss and -t) BEFORE specifying the input
     #       (-i), otherwise ffmpeg will decode everything and trim afterwards...
@@ -1389,7 +1698,7 @@ def read_opus_ffmpeg(
         cmd += f" -ar {force_opus_sampling_rate}"
         sampling_rate = force_opus_sampling_rate
     # Read audio samples directly as float32.
-    cmd += " -f f32le pipe:1"
+    cmd += " -f f32le -threads 1 pipe:1"
     # Actual audio reading.
     proc = run(cmd, shell=True, stdout=PIPE, stderr=PIPE)
     raw_audio = proc.stdout
@@ -1409,8 +1718,8 @@ def read_opus_ffmpeg(
                 f"Unknown channel description from ffmpeg: {channel_string}"
             )
     except ValueError as e:
-        raise ValueError(
-            f"{e}\nThe ffmpeg command for which the program failed is: '{cmd}'"
+        raise AudioLoadingError(
+            f"{e}\nThe ffmpeg command for which the program failed is: '{cmd}', error code: {proc.returncode}"
         )
     return audio, sampling_rate
 
@@ -1470,7 +1779,18 @@ def read_sph(
     cmd += f" {sph_path}"
 
     # Actual audio reading.
-    proc = BytesIO(run(cmd, shell=True, check=True, stdout=PIPE, stderr=PIPE).stdout)
+    try:
+        proc = BytesIO(
+            run(cmd, shell=True, check=True, stdout=PIPE, stderr=PIPE).stdout
+        )
+    except CalledProcessError as e:
+        if e.returncode == 127:
+            raise ValueError(
+                "It seems that 'sph2pipe' binary is not installed; "
+                "did you run 'lhotse install-sph2pipe'?"
+            )
+        else:
+            raise
 
     import soundfile as sf
 
@@ -1479,3 +1799,11 @@ def read_sph(
         audio = audio.reshape(1, -1) if sf_desc.channels == 1 else audio.T
 
     return audio, sampling_rate
+
+
+class AudioLoadingError(Exception):
+    pass
+
+
+class DurationMismatchError(Exception):
+    pass
