@@ -1,9 +1,13 @@
 from .adept import *
+from .aidatatang_200zh import *
 from .aishell import *
 from .aishell4 import *
+from .ali_meeting import *
 from .ami import *
+from .aspire import *
 from .babel import *
 from .broadcast_news import *
+from .bvcc import *
 from .callhome_egyptian import *
 from .callhome_english import *
 from .cmu_arctic import *
@@ -19,6 +23,7 @@ from .gale_mandarin import *
 from .gigaspeech import *
 from .heroico import *
 from .hifitts import *
+from .icsi import *
 from .l2_arctic import *
 from .libricss import *
 from .librimix import *
@@ -29,8 +34,13 @@ from .mls import *
 from .mtedx import *
 from .musan import *
 from .nsc import *
+from .peoples_speech import *
+from .rir_noise import *
+from .spgispeech import *
 from .switchboard import *
 from .tedlium import *
 from .timit import *
 from .vctk import *
+from .voxceleb import *
+from .wenet_speech import *
 from .yesno import *
