@@ -11,14 +11,14 @@ from typing_extensions import Literal
 
 from lhotse import CutSet
 from lhotse.cut import Cut
-from lhotse.dataset.sampling.base import CutSampler
-from lhotse.dataset.sampling.single_cut import SingleCutSampler
+from lhotse.dataset.sampling.base import CutSampler, SamplingDiagnostics
+from lhotse.dataset.sampling.simple import SimpleCutSampler
 
 
 class BucketingSampler(CutSampler):
     """
     Sorts the cuts in a :class:`CutSet` by their duration and puts them into similar duration buckets.
-    For each bucket, it instantiates a simpler sampler instance, e.g. :class:`SingleCutSampler`.
+    For each bucket, it instantiates a simpler sampler instance, e.g. :class:`SimpleCutSampler`.
 
     It behaves like an iterable that yields lists of strings (cut IDs).
     During iteration, it randomly selects one of the buckets to yield the batch from,
@@ -31,8 +31,8 @@ class BucketingSampler(CutSampler):
         >>> sampler = BucketingSampler(
         ...    cuts,
         ...    # BucketingSampler specific args
-        ...    sampler_type=SingleCutSampler, num_buckets=20,
-        ...    # Args passed into SingleCutSampler
+        ...    sampler_type=SimpleCutSampler, num_buckets=20,
+        ...    # Args passed into SimpleCutSampler
         ...    max_frames=20000
         ... )
 
@@ -50,7 +50,7 @@ class BucketingSampler(CutSampler):
     def __init__(
         self,
         *cuts: CutSet,
-        sampler_type: Type = SingleCutSampler,
+        sampler_type: Type = SimpleCutSampler,
         num_buckets: int = 10,
         bucket_method: Literal["equal_len", "equal_duration"] = "equal_len",
         drop_last: bool = False,
@@ -94,8 +94,8 @@ class BucketingSampler(CutSampler):
         self.cut_sets = cuts
         if self.cut_sets[0].is_lazy:
             warnings.warn(
-                "Lazy CutSet detected in BucketingSampler: this is not well supported yet, "
-                "and you might experience a potentially long lag while the buckets are being created."
+                "Lazy CutSet detected in BucketingSampler: we will read it into memory anyway. "
+                "Please use lhotse.dataset.DynamicBucketingSampler instead."
             )
 
         # Split data into buckets.
@@ -190,7 +190,7 @@ class BucketingSampler(CutSampler):
 
         Example:
             >>> cuts = CutSet(...)
-            ... sampler = SingleCutSampler(cuts, max_duration=100.0)
+            ... sampler = SimpleCutSampler(cuts, max_duration=100.0)
             ... # Retain only the cuts that have at least 1s and at most 20s duration.
             ... sampler.filter(lambda cut: 1.0 <= cut.duration <= 20.0)
         """
@@ -343,12 +343,13 @@ class BucketingSampler(CutSampler):
             if not depleted
         ]
 
+    @property
+    def diagnostics(self) -> SamplingDiagnostics:
+        return reduce(add, (bucket.diagnostics for bucket in self.bucket_samplers))
+
     def get_report(self) -> str:
         """Returns a string describing the statistics of the sampling process so far."""
-        total_diagnostics = reduce(
-            add, (bucket.diagnostics for bucket in self.bucket_samplers)
-        )
-        return total_diagnostics.get_report()
+        return self.diagnostics.get_report()
 
 
 def create_buckets_equal_len(
