@@ -4,7 +4,7 @@ from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
 from lhotse import CutSet
 from lhotse.cut import Cut
-from lhotse.dataset.sampling.base import CutSampler
+from lhotse.dataset.sampling.base import CutSampler, SamplingDiagnostics
 
 
 class ZipSampler(CutSampler):
@@ -22,8 +22,8 @@ class ZipSampler(CutSampler):
     Example::
 
         >>> sampler = ZipSampler(
-        ...     SingleCutSampler(cuts_corpusA, max_duration=250, shuffle=True),
-        ...     SingleCutSampler(cuts_corpusB, max_duration=100, shuffle=True),
+        ...     SimpleCutSampler(cuts_corpusA, max_duration=250, shuffle=True),
+        ...     SimpleCutSampler(cuts_corpusB, max_duration=100, shuffle=True),
         ... )
         >>> for cut in sampler:
         ...     pass  # profit
@@ -38,7 +38,7 @@ class ZipSampler(CutSampler):
             or return a tuple of CutSets. Setting this to ``False`` makes ZipSampler behave
             more like Python's ``zip`` function.
         """
-        super().__init__()
+        super().__init__(rank=0, world_size=1)
         self.samplers = samplers
         self.merge_batches = merge_batches
 
@@ -146,7 +146,7 @@ class ZipSampler(CutSampler):
             # different sources of cuts in any different way.
             #
             # Note: merging batches is tricky because the samplers can be either
-            # SingleCutSampler or CutPairsSampler, and we need to handle them differently.
+            # SimpleCutSampler or CutPairsSampler, and we need to handle them differently.
             cuts: List[Union[CutSet, Tuple[CutSet]]] = []
             for sampler in self.samplers:
                 batch = next(sampler)
@@ -195,16 +195,17 @@ class ZipSampler(CutSampler):
 
         Example:
             >>> cuts = CutSet(...)
-            ... sampler = SingleCutSampler(cuts, max_duration=100.0)
+            ... sampler = SimpleCutSampler(cuts, max_duration=100.0)
             ... # Retain only the cuts that have at least 1s and at most 20s duration.
             ... sampler.filter(lambda cut: 1.0 <= cut.duration <= 20.0)
         """
         for sampler in self.samplers:
             sampler.filter(predicate)
 
+    @property
+    def diagnostics(self) -> SamplingDiagnostics:
+        return reduce(add, (s.diagnostics for s in self.samplers))
+
     def get_report(self) -> str:
         """Returns a string describing the statistics of the sampling process so far."""
-        total_diagnostics = reduce(
-            add, (sampler.diagnostics for sampler in self.samplers)
-        )
-        return total_diagnostics.get_report()
+        return self.diagnostics.get_report()
