@@ -5,13 +5,15 @@ from .base import (
     Features,
     create_default_feature_extractor,
 )
-from .fbank import Fbank, FbankConfig
+from .fbank import TorchaudioFbank, TorchaudioFbankConfig
 from .io import (
     ChunkedLilcomHdf5Reader,
     ChunkedLilcomHdf5Writer,
     FeaturesReader,
     FeaturesWriter,
     KaldiReader,
+    LilcomChunkyReader,
+    LilcomChunkyWriter,
     LilcomFilesReader,
     LilcomFilesWriter,
     LilcomHdf5Reader,
@@ -25,7 +27,7 @@ from .io import (
     available_storage_backends,
     close_cached_file_handles,
 )
-from .kaldi.extractors import KaldiFbank, KaldiFbankConfig, KaldiMfcc, KaldiMfccConfig
+from .kaldi.extractors import Fbank, FbankConfig, Mfcc, MfccConfig
 from .kaldifeat import (
     KaldifeatFbank,
     KaldifeatFbankConfig,
@@ -33,6 +35,7 @@ from .kaldifeat import (
     KaldifeatMfccConfig,
 )
 from .librosa_fbank import LibrosaFbank, LibrosaFbankConfig
-from .mfcc import Mfcc, MfccConfig
+from .mfcc import TorchaudioMfcc, TorchaudioMfccConfig
 from .mixer import FeatureMixer
+from .opensmile import OpenSmileConfig, OpenSmileExtractor
 from .spectrogram import Spectrogram, SpectrogramConfig
