@@ -5,7 +5,7 @@ import warnings
 from abc import ABCMeta, abstractmethod
 from concurrent.futures.process import ProcessPoolExecutor
 from dataclasses import asdict, dataclass, is_dataclass
-from itertools import chain
+from itertools import chain, islice
 from math import isclose
 from pathlib import Path
 from typing import Any, Dict, Iterable, List, Optional, Sequence, Type, Union
@@ -17,6 +17,7 @@ from tqdm.auto import tqdm
 from lhotse.audio import Recording
 from lhotse.augmentation import AugmentFn
 from lhotse.features.io import FeaturesWriter, get_reader
+from lhotse.lazy import AlgorithmMixin
 from lhotse.serialization import Serializable, load_yaml, save_to_yaml
 from lhotse.utils import (
     Pathlike,
@@ -26,6 +27,7 @@ from lhotse.utils import (
     exactly_one_not_null,
     fastcopy,
     ifnone,
+    split_manifest_lazy,
     split_sequence,
     uuid4,
 )
@@ -132,8 +134,12 @@ class FeatureExtractor(metaclass=ABCMeta):
         )
 
     def extract_batch(
-        self, samples: Union[np.ndarray, Sequence[np.ndarray]], sampling_rate: int
-    ) -> Union[np.ndarray, List[np.ndarray]]:
+        self,
+        samples: Union[
+            np.ndarray, torch.Tensor, Sequence[np.ndarray], Sequence[torch.Tensor]
+        ],
+        sampling_rate: int,
+    ) -> Union[np.ndarray, torch.Tensor, List[np.ndarray], List[torch.Tensor]]:
         """
         Performs batch extraction. It is not guaranteed to be faster
         than :meth:`FeatureExtractor.extract` -- it depends on whether
@@ -148,6 +154,7 @@ class FeatureExtractor(metaclass=ABCMeta):
             This method *should* support variable length inputs.
         """
         input_is_list = False
+        input_is_torch = False
 
         if isinstance(samples, list):
             input_is_list = True
@@ -158,10 +165,17 @@ class FeatureExtractor(metaclass=ABCMeta):
             # The user passed an array/tensor of shape (num_samples,)
             samples = [samples.reshape(1, -1)]
 
+        if any(isinstance(x, torch.Tensor) for x in samples):
+            samples = [x.numpy() for x in samples]
+            input_is_torch = True
+
         result = []
         for item in samples:
             result.append(self.extract(item, sampling_rate=sampling_rate))
 
+        if input_is_torch:
+            result = [torch.from_numpy(x) for x in result]
+
         # If all items are of the same shape, concatenate
         if len(result) == 1:
             if input_is_list:
@@ -169,7 +183,10 @@ class FeatureExtractor(metaclass=ABCMeta):
             else:
                 return result[0]
         elif all(item.shape == result[0].shape for item in result[1:]):
-            return np.stack(result, axis=0)
+            if input_is_torch:
+                return torch.stack(result, dim=0)
+            else:
+                return np.stack(result, axis=0)
         else:
             return result
 
@@ -270,7 +287,7 @@ class FeatureExtractor(metaclass=ABCMeta):
             channels=channels if channels is not None else recording.channel_ids,
             # The start is relative to the beginning of the recording.
             start=offset,
-            duration=recording.duration,
+            duration=recording.duration if duration is not None else recording.duration,
             type=self.name,
             num_frames=feats.shape[0],
             num_features=feats.shape[1],
@@ -409,9 +426,10 @@ class Features:
     # to a directory holding files with feature matrices (exact semantics depend on storage_type).
     storage_path: str
 
-    # Storage key is either the key used to retrieve a feautre matrix from an archive like HDF5,
+    # Storage key is either the key used to retrieve a feature matrix from an archive like HDF5,
     # or the name of the file in a directory (exact semantics depend on the storage_type).
-    storage_key: str
+    # It can also be raw bytes for compressed arrays held in-memory.
+    storage_key: Union[str, bytes]
 
     # Information which recording and channels were used to extract the features.
     # When ``recording_id`` and ``channels`` are ``None``, it means that the
@@ -461,12 +479,58 @@ class Features:
             right_offset_frames=right_offset_frames,
         )
 
+    def move_to_memory(
+        self,
+        start: Seconds = 0,
+        duration: Optional[Seconds] = None,
+    ) -> "Features":
+        from lhotse.features.io import get_memory_writer
+
+        if self.storage_type in ("memory_lilcom", "memory_writer"):
+            return self  # nothing to do
+
+        arr = self.load(start=start, duration=duration)
+        if issubclass(arr.dtype.type, np.floating):
+            writer = get_memory_writer("memory_lilcom")()
+        else:
+            writer = get_memory_writer("memory_raw")()
+        data = writer.write("", arr)  # key is ignored by in memory writers
+        return fastcopy(
+            self,
+            # note: to understand why start is set to zero here, consider two cases:
+            # 1) this method moves the whole array to memory => the start was 0 anyway
+            # 2) this method moves a subset of the array to memory => the manifest is
+            #    now relative to the start of that subset, and since it describes the
+            #    whole subset, start=0 and duration=self.duration
+            start=0.0,
+            duration=ifnone(duration, self.duration),
+            num_frames=arr.shape[0],
+            storage_type=writer.name,
+            storage_key=data,
+            storage_path="",
+        )
+
     def with_path_prefix(self, path: Pathlike) -> "Features":
         return fastcopy(self, storage_path=str(Path(path) / self.storage_path))
 
     def to_dict(self) -> dict:
         return asdict_nonull(self)
 
+    def copy_feats(self, writer: FeaturesWriter) -> "Features":
+        """
+        Read the referenced feature array and save it using ``writer``.
+        Returns a copy of the manifest with updated fields related to the feature storage.
+        """
+        feats = self.load()
+        new_key = writer.write(self.storage_key, feats)
+        item = fastcopy(
+            self,
+            storage_type=writer.name,
+            storage_path=writer.storage_path,
+            storage_key=new_key,
+        )
+        return item
+
     @staticmethod
     def from_dict(data: dict) -> "Features":
         # The "storage_type" check is to ensure that the "data" dict actually contains
@@ -481,10 +545,34 @@ class Features:
             data["frame_shift"] = round(
                 data["duration"] / data["num_frames"], ndigits=3
             )
+        if (
+            "storage_key" in data
+            and "storage_type" in data
+            and "storage_path" not in data
+        ):
+            data["storage_path"] = None
         return Features(**data)
 
+    def __repr__(self):
+        return (
+            f"Features("
+            f"type='{self.type}', "
+            f"num_frames={self.num_frames}, "
+            f"num_features={self.num_features}, "
+            f"frame_shift={self.frame_shift}, "
+            f"sampling_rate={self.sampling_rate}, "
+            f"start={self.start}, "
+            f"duration={self.duration}, "
+            f"storage_type='{self.storage_type}', "
+            f"storage_path='{self.storage_path}', "
+            f"storage_key='{self.storage_key if isinstance(self.storage_key, str) else '<binary-data>'}', "
+            f"recording_id='{self.recording_id}', "
+            f"channels={self.channels}"
+            f")"
+        )
 
-class FeatureSet(Serializable, Sequence[Features]):
+
+class FeatureSet(Serializable, AlgorithmMixin):
     """
     Represents a feature manifest, and allows to read features for given recordings
     within particular channels and time ranges.
@@ -493,15 +581,27 @@ class FeatureSet(Serializable, Sequence[Features]):
     """
 
     def __init__(self, features: List[Features] = None) -> None:
-        self.features = sorted(ifnone(features, []))
+        self.features = ifnone(features, [])
+        if isinstance(self.features, list):
+            self.features = sorted(self.features)
 
     def __eq__(self, other: "FeatureSet") -> bool:
         return self.features == other.features
 
+    @property
+    def data(self) -> Union[Dict[str, Features], Iterable[Features]]:
+        """Alias property for ``self.features``"""
+        return self.features
+
+    def to_eager(self) -> "FeatureSet":
+        return FeatureSet(list(self))
+
     @staticmethod
     def from_features(features: Iterable[Features]) -> "FeatureSet":
         return FeatureSet(list(features))  # just for consistency with other *Sets
 
+    from_items = from_features
+
     @staticmethod
     def from_dicts(data: Iterable[dict]) -> "FeatureSet":
         return FeatureSet(
@@ -535,6 +635,28 @@ class FeatureSet(Serializable, Sequence[Features]):
             )
         ]
 
+    def split_lazy(self, output_dir: Pathlike, chunk_size: int) -> List["FeatureSet"]:
+        """
+        Splits a manifest (either lazily or eagerly opened) into chunks, each
+        with ``chunk_size`` items (except for the last one, typically).
+
+        In order to be memory efficient, this implementation saves each chunk
+        to disk in a ``.jsonl.gz`` format as the input manifest is sampled.
+
+        .. note:: For lowest memory usage, use ``load_manifest_lazy`` to open the
+            input manifest for this method.
+
+        :param it: any iterable of Lhotse manifests.
+        :param output_dir: directory where the split manifests are saved.
+            Each manifest is saved at: ``{output_dir}/{split_idx}.jsonl.gz``
+        :param chunk_size: the number of items in each chunk.
+        :return: a list of lazily opened chunk manifests.
+        """
+        return split_manifest_lazy(self, output_dir=output_dir, chunk_size=chunk_size)
+
+    def shuffle(self, *args, **kwargs):
+        raise NotImplementedError("FeatureSet does not support shuffling.")
+
     def subset(
         self, first: Optional[int] = None, last: Optional[int] = None
     ) -> "FeatureSet":
@@ -552,13 +674,12 @@ class FeatureSet(Serializable, Sequence[Features]):
 
         if first is not None:
             assert first > 0
-            if first > len(self):
+            out = FeatureSet.from_items(islice(self, first))
+            if len(out) < first:
                 logging.warning(
-                    f"FeatureSet has only {len(self)} items but first {first} required; "
-                    f"not doing anything."
+                    f"FeatureSet has only {len(out)} items but first {first} were requested."
                 )
-                return self
-            return FeatureSet.from_features(self.features[:first])
+            return out
 
         if last is not None:
             assert last > 0
@@ -657,6 +778,14 @@ class FeatureSet(Serializable, Sequence[Features]):
         features = feature_info.load(start=start, duration=duration)
         return features
 
+    def copy_feats(self, writer: FeaturesWriter) -> "FeatureSet":
+        """
+        For each manifest in this FeatureSet,
+        read the referenced feature array and save it using ``writer``.
+        Returns a copy of the manifest with updated fields related to the feature storage.
+        """
+        return FeatureSet.from_features(f.copy_feats(writer=writer) for f in self)
+
     def compute_global_stats(
         self, storage_path: Optional[Pathlike] = None
     ) -> Dict[str, np.ndarray]:
@@ -685,9 +814,6 @@ class FeatureSet(Serializable, Sequence[Features]):
     def __len__(self) -> int:
         return len(self.features)
 
-    def __add__(self, other: "FeatureSet") -> "FeatureSet":
-        return FeatureSet(features=self.features + other.features)
-
 
 class FeatureSetBuilder:
     """
