@@ -1,14 +1,17 @@
+import pickle
+import threading
 from abc import ABCMeta, abstractmethod
 from functools import lru_cache
 from math import ceil, floor
 from pathlib import Path
-from typing import List, Optional, Type
+from typing import List, Optional, Type, Union
 
 import lilcom
 import numpy as np
 
+from lhotse.array import Array, TemporalArray
 from lhotse.caching import dynamic_lru_cache
-from lhotse.utils import Pathlike, SmartOpen, is_module_available
+from lhotse.utils import Pathlike, Seconds, SmartOpen, is_module_available
 
 
 class FeaturesWriter(metaclass=ABCMeta):
@@ -31,15 +34,16 @@ class FeaturesWriter(metaclass=ABCMeta):
         it is stored in the features manifests (metadata) and used to automatically deduce
         the backend when loading the features.
 
-    Each ``FeaturesWriter`` can also be used as a context manager, as some implementations
+    Each :class:`.FeaturesWriter` can also be used as a context manager, as some implementations
     might need to free a resource after the writing is finalized. By default nothing happens
     in the context manager functions, and this can be modified by the inheriting subclasses.
 
-    Example:
-        with MyWriter('some/path') as storage:
-            extractor.extract_from_recording_and_store(recording, storage)
+    Example::
 
-    The features loading must be defined separately in a class inheriting from ``FeaturesReader``.
+        >>> with MyWriter('some/path') as storage:
+        ...     extractor.extract_from_recording_and_store(recording, storage)
+
+    The features loading must be defined separately in a class inheriting from :class:`FeaturesReader`.
     """
 
     @property
@@ -56,6 +60,62 @@ class FeaturesWriter(metaclass=ABCMeta):
     def write(self, key: str, value: np.ndarray) -> str:
         ...
 
+    def store_array(
+        self,
+        key: str,
+        value: np.ndarray,
+        frame_shift: Optional[Seconds] = None,
+        temporal_dim: Optional[int] = None,
+        start: Seconds = 0,
+    ) -> Union[Array, TemporalArray]:
+        """
+        Store a numpy array in the underlying storage and return a manifest
+        describing how to retrieve the data.
+
+        If the array contains a temporal dimension (e.g. it represents the
+        frame-level features, alignment, posteriors, etc. of an utterance)
+        then ``temporal_dim`` and ``frame_shift`` may be specified to enable
+        downstream padding, truncating, and partial reads of the array.
+
+        :param key: An ID that uniquely identifies the array.
+        :param value: The array to be stored.
+        :param frame_shift: Optional float, when the array has a temporal dimension
+            it indicates how much time has passed between the starts of consecutive frames
+            (expressed in seconds).
+        :param temporal_dim: Optional int, when the array has a temporal dimension,
+            it indicates which dim to interpret as temporal.
+        :param start: Float, when the array is temporal, it indicates what is the offset
+            of the array w.r.t. the start of recording. Useful for reading subsets
+            of an array when it represents something computed from long recordings.
+            Ignored for non-temporal arrays.
+        :return: A manifest of type :class:`~lhotse.array.Array` or
+            :class:`~lhotse.array.TemporalArray`, depending on the input arguments.
+        """
+        is_temporal = frame_shift is not None and temporal_dim is not None
+        if not is_temporal:
+            assert all(arg is None for arg in [frame_shift, temporal_dim]), (
+                "frame_shift and temporal_dim have to be both None or both set "
+                f"(got frame_shift={frame_shift}, temporal_dim={temporal_dim})."
+            )
+
+        storage_key = self.write(key, value)
+        array = Array(
+            storage_type=self.name,
+            storage_path=self.storage_path,
+            storage_key=storage_key,
+            shape=list(value.shape),
+        )
+
+        if not is_temporal:
+            return array
+
+        return TemporalArray(
+            array=array,
+            temporal_dim=temporal_dim,
+            frame_shift=frame_shift,
+            start=start,
+        )
+
     def __enter__(self):
         return self
 
@@ -212,7 +272,7 @@ class LilcomFilesWriter(FeaturesWriter):
 
     @property
     def storage_path(self) -> str:
-        return self.storage_path_
+        return str(self.storage_path_)
 
     def write(self, key: str, value: np.ndarray) -> str:
         # Introduce a sub-directory that starts with the first 3 characters of the key, that is typically
@@ -275,7 +335,7 @@ class NumpyFilesWriter(FeaturesWriter):
 
     @property
     def storage_path(self) -> str:
-        return self.storage_path_
+        return str(self.storage_path_)
 
     def write(self, key: str, value: np.ndarray) -> str:
         # Introduce a sub-directory that starts with the first 3 characters of the key, that is typically
@@ -309,6 +369,20 @@ def lookup_cache_or_open(storage_path: str):
     return h5py.File(storage_path, "r")
 
 
+@lru_cache(maxsize=None)
+def lookup_cache_or_open_regular_file(storage_path: str):
+    """
+    Helper internal function used in "fast" file readers.
+    It opens regular files and keeps their handles open in a global program cache
+    to avoid excessive amount of syscalls when the Reader class is instantiated
+    and destroyed in a loop repeatedly (frequent use-case).
+
+    The file handles can be freed at any time by calling ``close_cached_file_handles()``.
+    """
+    f = open(storage_path, "rb")
+    return f
+
+
 @lru_cache(maxsize=None)
 def lookup_chunk_size(h5_file_handle) -> int:
     """
@@ -320,6 +394,7 @@ def lookup_chunk_size(h5_file_handle) -> int:
 
 def close_cached_file_handles() -> None:
     """Closes the cached file handles in ``lookup_cache_or_open`` (see its docs for more details)."""
+    lookup_cache_or_open_regular_file.cache_clear()
     lookup_cache_or_open.cache_clear()
     lookup_chunk_size.cache_clear()
 
@@ -382,7 +457,7 @@ class NumpyHdf5Writer(FeaturesWriter):
 
     @property
     def storage_path(self) -> str:
-        return self.storage_path_
+        return str(self.storage_path_)
 
     def write(self, key: str, value: np.ndarray) -> str:
         self.hdf.create_dataset(key, data=value)
@@ -530,13 +605,14 @@ class ChunkedLilcomHdf5Reader(FeaturesReader):
             right_chunk_idx = None
 
         # Read, decode, concat
-        arr = np.concatenate(
-            [
-                lilcom.decompress(data.tobytes())
-                for data in self.hdf[key][left_chunk_idx:right_chunk_idx]
-            ],
-            axis=0,
-        )
+        decompressed_chunks = [
+            lilcom.decompress(data.tobytes())
+            for data in self.hdf[key][left_chunk_idx:right_chunk_idx]
+        ]
+        if decompressed_chunks:
+            arr = np.concatenate(decompressed_chunks, axis=0)
+        else:
+            arr = np.array([])
 
         # Determine what piece of decoded data should be returned;
         # we offset the input offsets by left_chunk_idx * chunk_size.
@@ -631,6 +707,163 @@ class ChunkedLilcomHdf5Writer(FeaturesWriter):
         self.close()
 
 
+"""
+Lilcom-compressed numpy arrays, stored in my own custom format file, that store chunked features.
+They are suitable for storing features for long recordings since they are able to
+retrieve small chunks instead of full matrices.
+"""
+
+CHUNKY_FORMAT_CHUNK_SIZE = 500  # constant
+
+
+@register_reader
+class LilcomChunkyReader(FeaturesReader):
+    """
+    Reads lilcom-compressed numpy arrays from a binary file with chunked lilcom storage.
+    Each feature matrix is stored in an array of chunks - binary data compressed with lilcom.
+    Upon reading, we check how many chunks need to be retrieved to avoid excessive I/O.
+
+    ``storage_path`` corresponds to the binary file path.
+
+    ``storage_key`` for each utterance is a comma separated list of offsets in the file.
+    The first number is the offset for the whole array,
+    and the following numbers are relative offsets for each chunk.
+    These offsets are relative to the previous chunk start.
+    """
+
+    name = "lilcom_chunky"
+    CHUNK_SIZE = CHUNKY_FORMAT_CHUNK_SIZE
+
+    def __init__(self, storage_path: Pathlike, *args, **kwargs):
+        super().__init__()
+        self.file = lookup_cache_or_open_regular_file(storage_path)
+        self.lock = threading.Lock()
+
+    @dynamic_lru_cache
+    def read(
+        self,
+        key: str,
+        left_offset_frames: int = 0,
+        right_offset_frames: Optional[int] = None,
+    ) -> np.ndarray:
+        # First, determine which range of chunks need to be read.
+        left_chunk_idx = floor(left_offset_frames / self.CHUNK_SIZE)
+        if right_offset_frames is not None:
+            # Note: +1 is to include the end of the last chunk
+            right_chunk_idx = ceil(right_offset_frames / self.CHUNK_SIZE) + 1
+        else:
+            right_chunk_idx = None
+
+        chunk_offsets = list(map(int, key.split(",")))
+        chunk_offsets = np.cumsum(chunk_offsets)
+        chunk_offsets = chunk_offsets[left_chunk_idx:right_chunk_idx]
+
+        chunk_data = []
+        for offset, end in pairwise(chunk_offsets):
+            # We need to use locks to avoid race conditions between seek
+            # and read in multi-threaded reads.
+            with self.lock:
+                self.file.seek(offset)
+                chunk_data.append(self.file.read(end - offset))
+
+        # Read, decode, concat
+        decompressed_chunks = [lilcom.decompress(data) for data in chunk_data]
+        if decompressed_chunks:
+            arr = np.concatenate(decompressed_chunks, axis=0)
+        else:
+            arr = np.array([])
+
+        # Determine what piece of decoded data should be returned;
+        # we offset the input offsets by left_chunk_idx * chunk_size.
+        shift_frames = self.CHUNK_SIZE * left_chunk_idx
+        left_offset_shift = left_offset_frames - shift_frames
+        if right_offset_frames is not None:
+            right_offset_shift = right_offset_frames - shift_frames
+        else:
+            right_offset_shift = None
+
+        return arr[left_offset_shift:right_offset_shift]
+
+
+@register_writer
+class LilcomChunkyWriter(FeaturesWriter):
+    """
+    Writes lilcom-compressed numpy arrays to a binary file with chunked lilcom storage.
+    Each feature matrix is stored in an array of chunks - binary data compressed with lilcom.
+    Upon reading, we check how many chunks need to be retrieved to avoid excessive I/O.
+
+    ``storage_path`` corresponds to the binary file path.
+
+    ``storage_key`` for each utterance is a comma separated list of offsets in the file.
+    The first number is the offset for the whole array,
+    and the following numbers are relative offsets for each chunk.
+    These offsets are relative to the previous chunk start.
+    """
+
+    name = "lilcom_chunky"
+    CHUNK_SIZE = CHUNKY_FORMAT_CHUNK_SIZE
+
+    def __init__(
+        self,
+        storage_path: Pathlike,
+        tick_power: int = -5,
+        mode: str = "wb",
+        *args,
+        **kwargs,
+    ):
+        """
+        :param storage_path: Path under which we'll create the binary file.
+        :param tick_power: Determines the lilcom compression accuracy;
+            the input will be compressed to integer multiples of 2^tick_power.
+        :param chunk_size: How many frames to store per chunk.
+            Too low a number will require many reads for long feature matrices,
+            too high a number will require to read more redundant data.
+        :param mode: Modes, one of: "w" (write) or "a" (append); can be "wb" and "ab", "b" is implicit
+        """
+        super().__init__()
+
+        if "b" not in mode:
+            mode = mode + "b"
+        assert mode == "wb" or "ab"
+
+        # ".lca" -> "lilcom chunky archive"
+        self.storage_path_ = Path(storage_path).with_suffix(".lca")
+        self.tick_power = tick_power
+        self.file = open(self.storage_path, mode=mode)
+        self.curr_offset = self.file.tell()
+
+    @property
+    def storage_path(self) -> str:
+        return str(self.storage_path_)
+
+    def write(self, key: str, value: np.ndarray) -> str:
+        from lhotse.features.compression import lilcom_compress_chunked
+
+        serialized_feats = lilcom_compress_chunked(
+            value, tick_power=self.tick_power, chunk_size=self.CHUNK_SIZE
+        )
+        offsets = [self.curr_offset]
+        for idx, feat in enumerate(serialized_feats):
+            nbytes = self.file.write(feat)
+            offsets.append(nbytes)
+            self.curr_offset += nbytes
+
+        # Returns keys like: "14601,31,23,42".
+        # The first number is the offset for the whole array,
+        # and the following numbers are relative offsets for each chunk.
+        # These offsets are relative to the previous chunk start.
+        return ",".join(map(str, offsets))
+
+    def close(self) -> None:
+        self.file.close()
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.close()
+
+
 """
 Lilcom-compressed URL writers
 """
@@ -718,26 +951,28 @@ Kaldi-compatible feature reader
 @register_reader
 class KaldiReader(FeaturesReader):
     """
-    Reads Kaldi's "feats.scp" file using kaldiio.
+    Reads Kaldi's "feats.scp" file using kaldi_native_io.
     ``storage_path`` corresponds to the path to ``feats.scp``.
     ``storage_key`` corresponds to the utterance-id in Kaldi.
 
     .. caution::
-        Requires ``kaldiio`` to be installed (``pip install kaldiio``).
+        Requires ``kaldi_native_io`` to be installed (``pip install kaldi_native_io``).
     """
 
     name = "kaldiio"
 
     def __init__(self, storage_path: Pathlike, *args, **kwargs):
-        if not is_module_available("kaldiio"):
+        if not is_module_available("kaldi_native_io"):
             raise ValueError(
-                "To read Kaldi feats.scp, please 'pip install kaldiio' first."
+                "To read Kaldi feats.scp, please 'pip install kaldi_native_io' first."
             )
-        import kaldiio
+        import kaldi_native_io
 
         super().__init__()
         self.storage_path = storage_path
-        self.storage = kaldiio.load_scp(str(self.storage_path))
+        self.storage = kaldi_native_io.RandomAccessFloatMatrixReader(
+            f"scp:{self.storage_path}"
+        )
 
     @dynamic_lru_cache
     def read(
@@ -746,5 +981,191 @@ class KaldiReader(FeaturesReader):
         left_offset_frames: int = 0,
         right_offset_frames: Optional[int] = None,
     ) -> np.ndarray:
-        arr = self.storage[key]
+        arr = np.copy(self.storage[key])
         return arr[left_offset_frames:right_offset_frames]
+
+
+@register_writer
+class KaldiWriter(FeaturesWriter):
+    """
+    Write data to Kaldi's "feats.scp" and "feats.ark" files using kaldi_native_io.
+    ``storage_path`` corresponds to a directory where we'll create "feats.scp"
+    and "feats.ark" files.
+    ``storage_key`` corresponds to the utterance-id in Kaldi.
+
+    The following ``compression_method`` values are supported by kaldi_native_io::
+
+        kAutomaticMethod = 1
+        kSpeechFeature = 2
+        kTwoByteAuto = 3
+        kTwoByteSignedInteger = 4
+        kOneByteAuto = 5
+        kOneByteUnsignedInteger = 6
+        kOneByteZeroOne = 7
+
+    .. note:: Setting compression_method works only with 2D arrays.
+
+    Example::
+
+        >>> data = np.random.randn(131, 80)
+        >>> with KaldiWriter('featdir') as w:
+        ...     w.write('utt1', data)
+        >>> reader = KaldiReader('featdir/feats.scp')
+        >>> read_data = reader.read('utt1')
+        >>> np.testing.assert_equal(data, read_data)
+
+    .. caution::
+        Requires ``kaldi_native_io`` to be installed (``pip install kaldi_native_io``).
+    """
+
+    name = "kaldiio"
+
+    def __init__(
+        self,
+        storage_path: Pathlike,
+        compression_method: int = 1,
+        *args,
+        **kwargs,
+    ):
+        if not is_module_available("kaldi_native_io"):
+            raise ValueError(
+                "To read Kaldi feats.scp, please 'pip install kaldi_native_io' first."
+            )
+        import kaldi_native_io
+
+        super().__init__()
+        self.storage_dir = Path(storage_path)
+        self.storage_dir.mkdir(parents=True, exist_ok=True)
+        self.storage_path_ = str(self.storage_dir / "feats.scp")
+        self.storage = kaldi_native_io.CompressedMatrixWriter(
+            f"ark,scp:{self.storage_dir}/feats.ark,{self.storage_dir}/feats.scp"
+        )
+        self.compression_method = kaldi_native_io.CompressionMethod(compression_method)
+
+    @property
+    def storage_path(self) -> str:
+        return self.storage_path_
+
+    def write(self, key: str, value: np.ndarray) -> str:
+        self.storage.write(key, value, self.compression_method)
+        return key
+
+    def close(self) -> None:
+        return self.storage.close()
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.close()
+
+
+"""
+In-memory reader/writer
+"""
+
+
+def get_memory_writer(name: str):
+    assert "memory" in name
+    return get_writer(name)
+
+
+@register_reader
+class MemoryLilcomReader(FeaturesReader):
+    """ """
+
+    name = "memory_lilcom"
+
+    def __init__(self, *args, **kwargs):
+        pass
+
+    @dynamic_lru_cache
+    def read(
+        self,
+        raw_data: bytes,
+        left_offset_frames: int = 0,
+        right_offset_frames: Optional[int] = None,
+    ) -> np.ndarray:
+        arr = lilcom.decompress(raw_data)
+        return arr[left_offset_frames:right_offset_frames]
+
+
+@register_writer
+class MemoryLilcomWriter(FeaturesWriter):
+    """ """
+
+    name = "memory_lilcom"
+
+    def __init__(self, *args, **kwargs):
+        pass
+
+    @property
+    def storage_path(self) -> None:
+        return None
+
+    def write(self, key: str, value: np.ndarray) -> bytes:
+        return lilcom.compress(value)
+
+    def close(self) -> None:
+        pass
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        pass
+
+
+@register_reader
+class MemoryRawReader(FeaturesReader):
+    """ """
+
+    name = "memory_raw"
+
+    def __init__(self, *args, **kwargs):
+        pass
+
+    @dynamic_lru_cache
+    def read(
+        self,
+        raw_data: bytes,
+        left_offset_frames: int = 0,
+        right_offset_frames: Optional[int] = None,
+    ) -> np.ndarray:
+        arr = pickle.loads(raw_data)
+        return arr[left_offset_frames:right_offset_frames]
+
+
+@register_writer
+class MemoryRawWriter(FeaturesWriter):
+    """ """
+
+    name = "memory_raw"
+
+    def __init__(self, *args, **kwargs):
+        pass
+
+    @property
+    def storage_path(self) -> None:
+        return None
+
+    def write(self, key: str, value: np.ndarray) -> bytes:
+        return pickle.dumps(value)
+
+    def close(self) -> None:
+        pass
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        pass
+
+
+def pairwise(iterable):
+    "s -> (s0,s1), (s1,s2), (s2, s3), ..."
+    from itertools import tee
+
+    a, b = tee(iterable)
+    next(b, None)
+    return zip(a, b)
