@@ -4,7 +4,7 @@ from concurrent.futures import ProcessPoolExecutor
 from pathlib import Path
 from typing import Any, Dict, Optional, Tuple
 
-from lhotse import CutSet, FeatureSet, Features, Seconds
+from lhotse import FeatureSet, Features, Seconds
 from lhotse.audio import AudioSource, Recording, RecordingSet
 from lhotse.audio import audioread_info
 from lhotse.supervision import SupervisionSegment, SupervisionSet
@@ -28,19 +28,17 @@ def get_duration(
     """
     path = str(path)
     if path.strip().endswith("|"):
-        if not is_module_available("kaldiio"):
+        if not is_module_available("kaldi_native_io"):
             raise ValueError(
                 "To read Kaldi's data dir where wav.scp has 'pipe' inputs, "
-                "please 'pip install kaldiio' first."
+                "please 'pip install kaldi_native_io' first."
             )
-        from kaldiio import load_mat
+        import kaldi_native_io
 
-        # Note: kaldiio.load_mat returns
-        # (sampling_rate: int, samples: 1-D np.array[int])
-        sampling_rate, samples = load_mat(path)
-        assert len(samples.shape) == 1
-        duration = samples.shape[0] / sampling_rate
-        return duration
+        wave_info = kaldi_native_io.read_wave_info(path)
+        assert wave_info.num_channels == 1, wave_info.num_channels
+
+        return wave_info.duration
     try:
         # Try to parse the file using pysoundfile first.
         import soundfile
@@ -135,14 +133,14 @@ def load_kaldi_data_dir(
 
     feature_set = None
     feats_scp = path / "feats.scp"
-    if feats_scp.exists() and is_module_available("kaldiio"):
+    if feats_scp.exists() and is_module_available("kaldi_native_io"):
         if frame_shift is not None:
-            import kaldiio
+            import kaldi_native_io
             from lhotse.features.io import KaldiReader
 
             feature_set = FeatureSet.from_features(
                 Features(
-                    type="kaldiio",
+                    type="kaldi_native_io",
                     num_frames=mat.shape[0],
                     num_features=mat.shape[1],
                     frame_shift=frame_shift,
@@ -157,7 +155,9 @@ def load_kaldi_data_dir(
                     else utt_id,
                     channels=0,
                 )
-                for utt_id, mat in kaldiio.load_scp_sequential(str(feats_scp))
+                for utt_id, mat in kaldi_native_io.SequentialFloatMatrixReader(
+                    f"scp:{feats_scp}"
+                )
             )
         else:
             warnings.warn(
@@ -174,6 +174,7 @@ def export_to_kaldi(
     supervisions: SupervisionSet,
     output_dir: Pathlike,
     map_underscores_to: Optional[str] = None,
+    prefix_spk_id: Optional[bool] = False,
 ):
     """
     Export a pair of ``RecordingSet`` and ``SupervisionSet`` to a Kaldi data
@@ -188,6 +189,8 @@ def export_to_kaldi(
     :param output_dir: path where the Kaldi-style data directory will be created.
     :param map_underscores_to: optional string with which we will replace all
         underscores. This helps avoid issues with Kaldi data dir sorting.
+    :param prefix_spk_id: add speaker_id as a prefix of utterance_id (this is to
+        ensure correct sorting inside files which is required by Kaldi)
     """
     output_dir = Path(output_dir)
     output_dir.mkdir(parents=True, exist_ok=True)
@@ -206,11 +209,8 @@ def export_to_kaldi(
             )
         )
 
-    # Create a simple CutSet that ties together
-    # the recording <-> supervision information.
-    cuts = CutSet.from_manifests(
-        recordings=recordings, supervisions=supervisions
-    ).trim_to_supervisions()
+    if prefix_spk_id:
+        supervisions = supervisions.map(lambda s: fastcopy(s, id=f"{s.speaker}-{s.id}"))
 
     if all(r.num_channels == 1 for r in recordings):
         # if all the recordings are single channel, we won't add
@@ -231,8 +231,8 @@ def export_to_kaldi(
         # segments
         save_kaldi_text_mapping(
             data={
-                cut.supervisions[0].id: f"{cut.recording_id} {cut.start} {cut.end}"
-                for cut in cuts
+                sup.id: f"{sup.recording_id} {sup.start} {sup.end}"
+                for sup in supervisions
             },
             path=output_dir / "segments",
         )
@@ -258,10 +258,8 @@ def export_to_kaldi(
         # segments
         save_kaldi_text_mapping(
             data={
-                cut.supervisions[
-                    0
-                ].id: f"{cut.recording_id}_{cut.channel} {cut.start} {cut.end}"
-                for cut in cuts
+                sup.id: f"{sup.recording_id} {sup.start} {sup.end}"
+                for sup in supervisions
             },
             path=output_dir / "segments",
         )
@@ -276,29 +274,29 @@ def export_to_kaldi(
         )
     # text
     save_kaldi_text_mapping(
-        data={cut.supervisions[0].id: cut.supervisions[0].text for cut in cuts},
+        data={sup.id: sup.text for sup in supervisions},
         path=output_dir / "text",
     )
     # utt2spk
     save_kaldi_text_mapping(
-        data={cut.supervisions[0].id: cut.supervisions[0].speaker for cut in cuts},
+        data={sup.id: sup.speaker for sup in supervisions},
         path=output_dir / "utt2spk",
     )
     # utt2dur
     save_kaldi_text_mapping(
-        data={cut.supervisions[0].id: cut.duration for cut in cuts},
+        data={sup.id: sup.duration for sup in supervisions},
         path=output_dir / "utt2dur",
     )
     # utt2lang [optional]
     if all(s.language is not None for s in supervisions):
         save_kaldi_text_mapping(
-            data={cut.supervisions[0].id: cut.supervisions[0].language for cut in cuts},
+            data={sup.id: sup.language for sup in supervisions},
             path=output_dir / "utt2lang",
         )
     # utt2gender [optional]
     if all(s.gender is not None for s in supervisions):
         save_kaldi_text_mapping(
-            data={cut.supervisions[0].id: cut.supervisions[0].gender for cut in cuts},
+            data={sup.id: sup.gender for sup in supervisions},
             path=output_dir / "utt2gender",
         )
 
@@ -333,9 +331,11 @@ def make_wavscp_channel_string_map(
             raise ValueError(
                 "Command audio multichannel sources are not supported yet."
             )
-        return f"{source.source} |"
+        return {0: f"{source.source} |"}
     elif source.type == "file":
-        if Path(source.source).suffix == ".wav":
+        if Path(source.source).suffix == ".wav" and len(source.channels) == 1:
+            # Note: for single-channel waves, we don't need to invoke ffmpeg; but
+            #       for multi-channel waves, Kaldi is going to complain.
             audios = dict()
             for channel in source.channels:
                 audios[channel] = source.source
@@ -348,15 +348,16 @@ def make_wavscp_channel_string_map(
             for channel in source.channels:
                 audios[
                     channel
-                ] = f"sph2pipe {source.source} -f wav -c {channel+1} -p | ffmpeg -i pipe:0 -ar {sampling_rate} -f wav  pipe:1 |"
+                ] = f"sph2pipe {source.source} -f wav -c {channel+1} -p | ffmpeg -threads 1 -i pipe:0 -ar {sampling_rate} -f wav -threads 1 pipe:1 |"
 
             return audios
         else:
+            # Handles non-WAVE audio formats and multi-channel WAVEs.
             audios = dict()
             for channel in source.channels:
                 audios[
                     channel
-                ] = f"ffmpeg -i {source.source} -ar {sampling_rate} -map_channel 0.0.{channel}  -f wav pipe:1 |"
+                ] = f"ffmpeg -threads 1 -i {source.source} -ar {sampling_rate} -map_channel 0.0.{channel}  -f wav -threads 1 pipe:1 |"
             return audios
 
     else:
