@@ -1,14 +1,16 @@
+from collections import defaultdict
 import logging
 from math import isclose
 from typing import Any, Callable, Dict, Optional, Tuple, Union
 
 import numpy as np
 
+from lhotse.array import Array, TemporalArray
 from lhotse.audio import Recording, RecordingSet
 from lhotse.cut import Cut, CutSet, MixedCut, PaddingCut
 from lhotse.features import FeatureSet, Features
 from lhotse.supervision import SupervisionSegment, SupervisionSet
-from lhotse.utils import compute_num_frames
+from lhotse.utils import compute_num_frames, overlaps
 
 _VALIDATORS: Dict[str, Callable] = {}
 
@@ -86,6 +88,12 @@ def validate_recordings_and_supervisions(
         recordings = RecordingSet.from_recordings([recordings])
     if isinstance(supervisions, SupervisionSegment):
         supervisions = SupervisionSet.from_segments([supervisions])
+
+    if recordings.is_lazy:
+        recordings = RecordingSet.from_recordings(iter(recordings))
+    if supervisions.is_lazy:
+        supervisions = SupervisionSet.from_segments(iter(supervisions))
+
     validate(recordings, read_data=read_data)
     validate(supervisions)
     # Errors
@@ -138,11 +146,14 @@ def remove_missing_recordings_and_supervisions(
         )
     only_in_supervisions = recording_ids_in_sups - recording_ids
     if only_in_supervisions:
+        n_orig_sups = len(supervisions)
         supervisions = supervisions.filter(
             lambda s: s.recording_id not in only_in_supervisions
         )
+        n_removed_sups = n_orig_sups - len(supervisions)
         logging.warning(
-            f"Removed {len(only_in_supervisions)} supervisions with no corresponding recordings."
+            f"Removed {n_removed_sups} supervisions with no corresponding recordings "
+            f"(for a total of {len(only_in_supervisions)} recording IDs)."
         )
     return recordings, supervisions
 
@@ -154,6 +165,9 @@ def trim_supervisions_to_recordings(
     Return a new :class:`~lhotse.supervision.SupervisionSet` with supervisions that are
     not exceeding the duration of their corresponding :class:`~lhotse.audio.Recording`.
     """
+    if recordings.is_lazy:
+        recordings = RecordingSet.from_recordings(iter(recordings))
+
     sups = []
     removed = 0
     trimmed = 0
@@ -212,11 +226,31 @@ def validate_recording(r: Recording, read_data: bool = False) -> None:
 
 
 @register_validator
-def validate_supervision(s: SupervisionSegment, **kwargs) -> None:
+def validate_supervision(
+    s: SupervisionSegment, read_data: bool = False, **kwargs
+) -> None:
     assert (
         s.duration > 0
     ), f"Supervision {s.id}: duration has to be greater than 0 (is {s.duration})"
 
+    # Conditions related to custom fields
+    if s.custom is not None:
+        assert isinstance(
+            s.custom, dict
+        ), f"SupervisionSegment {s.id}: custom field has to be set to a dict or None."
+        for key, value in s.custom.items():
+            if isinstance(value, Array):
+                validate_array(value, read_data=read_data)
+            elif isinstance(value, TemporalArray):
+                validate_temporal_array(value, read_data=read_data)
+                if not isclose(s.duration, value.duration):
+                    logging.warning(
+                        f"SupervisionSegment {s.id}: possibly mismatched "
+                        f"duration between supervision ({s.duration}s) and temporal array "
+                        f"in custom field '{key}' (num_frames={value.num_frames} * "
+                        f"frame_shift={value.frame_shift} == duration={value.duration})."
+                    )
+
 
 @register_validator
 def validate_features(
@@ -266,6 +300,27 @@ def validate_features(
         ), f"Features: expected num_features: {f.num_features}, actual: {n_ft}"
 
 
+@register_validator
+def validate_array(arr: Array, read_data: bool = False) -> None:
+    if read_data:
+        data = arr.load()
+        assert data.shape == arr.shape
+
+
+@register_validator
+def validate_temporal_array(arr: TemporalArray, read_data: bool = False) -> None:
+    assert arr.temporal_dim >= 0, "TemporalArray: temporal_dim cannot be negative."
+    assert arr.temporal_dim < arr.ndim, (
+        f"TemporalArray: temporal_dim {arr.temporal_dim} "
+        f"canot be greater than ndim {arr.ndim}."
+    )
+    assert arr.frame_shift > 0, "TemporalArray: frame_shift must be positive."
+    assert arr.start >= 0, "TemporalArray: start must be non-negative."
+    if read_data:
+        data = arr.load()
+        assert data.shape == arr.shape
+
+
 @register_validator
 def validate_cut(c: Cut, read_data: bool = False) -> None:
     # Validate MixedCut
@@ -336,6 +391,30 @@ def validate_cut(c: Cut, read_data: bool = False) -> None:
             f"(expected {c.channel}, supervision has {s.channel})"
         )
 
+    # Conditions related to custom fields
+    if c.custom is not None:
+        assert isinstance(
+            c.custom, dict
+        ), f"MonoCut {c.id}: custom field has to be set to a dict or None."
+        for key, value in c.custom.items():
+            if isinstance(value, Array):
+                validate_array(value, read_data=read_data)
+            elif isinstance(value, TemporalArray):
+                validate_temporal_array(value, read_data=read_data)
+                if not isclose(c.duration, value.duration):
+                    logging.warning(
+                        f"MonoCut {c.id}: possibly mismatched "
+                        f"duration between cut ({c.duration}s) and temporal array "
+                        f"in custom field '{key}' (num_frames={value.num_frames} * "
+                        f"frame_shift={value.frame_shift} == duration={value.duration})."
+                    )
+                assert overlaps(c, value), (
+                    f"MonoCut {c.id}: TemporalArray at custom field '{key}' "
+                    "seems to have incorrect start time (the array with time span "
+                    f"[{value.start}s - {value.end}s] does not overlap with cut "
+                    f"with time span [{c.start}s - {c.end}s])."
+                )
+
 
 @register_validator
 def validate_recording_set(recordings: RecordingSet, read_data: bool = False) -> None:
@@ -355,6 +434,20 @@ def validate_supervision_set(supervisions: SupervisionSet, **kwargs) -> None:
     for s in supervisions:
         validate_supervision(s)
 
+    # Catch errors in data preparation:
+    # - more than one supervision for a given recording starts at 0 (in a given channel)
+    supervisions._index_by_recording_id_and_cache()
+    for rid, sups in supervisions._segments_by_recording_id.items():
+        cntr_per_channel = defaultdict(int)
+        for s in sups:
+            cntr_per_channel[s.channel] += int(s.start == 0)
+        for channel, count in cntr_per_channel.items():
+            if count > 1:
+                logging.warning(
+                    f"SupervisionSet contains {count} supervisions that start at 0 for recording {rid} "
+                    f"(channel {channel}). Did you forget to set supervision start times?"
+                )
+
 
 @register_validator
 def validate_feature_set(features: FeatureSet, read_data: bool = False) -> None:
