@@ -221,20 +221,22 @@ def download_ami(
     force_download: Optional[bool] = False,
     url: Optional[str] = "http://groups.inf.ed.ac.uk/ami",
     mic: Optional[str] = "ihm",
-) -> None:
+) -> Path:
     """
     Download AMI audio and annotations for provided microphone setting.
-    :param target_dir: Pathlike, the path to store the data.
-    :param annotations: Pathlike (default = None), path to save annotations zip file
-    :param force_download: bool (default = False), if True, download even if file is present.
-    :param url: str (default = 'http://groups.inf.ed.ac.uk/ami'), AMI download URL.
-    :param mic: str {'ihm','ihm-mix','sdm','mdm'}, type of mic setting.
 
     Example usage:
     1. Download AMI data for IHM mic setting:
     >>> download_ami(mic='ihm')
     2. Download AMI data for IHM-mix mic setting, and use existing annotations:
     >>> download_ami(mic='ihm-mix', annotations='/path/to/existing/annotations.zip')
+
+    :param target_dir: Pathlike, the path to store the data.
+    :param annotations: Pathlike (default = None), path to save annotations zip file
+    :param force_download: bool (default = False), if True, download even if file is present.
+    :param url: str (default = 'http://groups.inf.ed.ac.uk/ami'), AMI download URL.
+    :param mic: str {'ihm','ihm-mix','sdm','mdm'}, type of mic setting.
+    :return: the path to downloaded and extracted directory with data.
     """
     target_dir = Path(target_dir)
 
@@ -250,11 +252,13 @@ def download_ami(
 
     if annotations.exists():
         logging.info(f"Skip downloading annotations as they exist in: {annotations}")
-        return
+        return target_dir
     annotations_url = f"{url}/AMICorpusAnnotations/ami_public_manual_1.6.2.zip"
     if force_download or not annotations.is_file():
         urllib.request.urlretrieve(annotations_url, filename=annotations)
 
+    return target_dir
+
 
 class AmiSegmentAnnotation(NamedTuple):
     text: str
@@ -265,7 +269,7 @@ class AmiSegmentAnnotation(NamedTuple):
 
 
 def parse_ami_annotations(
-    annotations_dir: Pathlike, normalize_text: bool = True
+    annotations_dir: Pathlike, normalize: str = "upper"
 ) -> Dict[str, List[SupervisionSegment]]:
     annotations = defaultdict(dict)
 
@@ -298,9 +302,7 @@ def parse_ami_annotations(
             for word in tree.getroot():
                 if word.tag != "w" or "punc" in word.attrib:
                     continue
-                wid_to_word[word.attrib["{http://nite.sourceforge.net/}id"]] = (
-                    word.text.upper() if normalize_text else word.text
-                )
+                wid_to_word[word.attrib["{http://nite.sourceforge.net/}id"]] = word.text
 
     def _parse_href(href, wid_to_word):
         # The href argument is originally a string of the form "ES2002b.B.words.xml#id(ES2002b.B.words0)..id(ES2002b.B.words4)".
@@ -339,23 +341,48 @@ def parse_ami_annotations(
                     continue
                 start_time = float(seg.attrib["transcriber_start"])
                 end_time = float(seg.attrib["transcriber_end"])
-                assert len(seg.getchildren()) == 1, "Multiple child segments found"
-                seg_child = seg.getchildren()[0]
+                assert len(seg) == 1, "Multiple child segments found"
+                seg_child = next(iter(seg))
                 if "href" in seg_child.attrib:
                     text = _parse_href(seg_child.attrib["href"], wid_to_word)
-                annotations[key].append(
-                    AmiSegmentAnnotation(
-                        text=text.upper() if normalize_text else text,
-                        speaker=spk,
-                        gender=spk[0],
-                        start_time=start_time,
-                        end_time=end_time,
+                    text = normalize_text(text, normalize)
+                if len(text) > 0:
+                    annotations[key].append(
+                        AmiSegmentAnnotation(
+                            text=text,
+                            speaker=spk,
+                            gender=spk[0],
+                            start_time=start_time,
+                            end_time=end_time,
+                        )
                     )
-                )
 
     return annotations
 
 
+def normalize_text(text: str, normalize: str = "upper") -> str:
+    if normalize == "none":
+        return text
+    elif normalize == "upper":
+        return text.upper()
+    elif normalize == "kaldi":
+        # Kaldi style text normalization
+        import re
+
+        # convert text to uppercase
+        text = text.upper()
+        # remove punctuations
+        text = re.sub(r"[^A-Z0-9']+", " ", text)
+        # remove multiple spaces
+        text = re.sub(r"\s+", " ", text)
+        # apply few exception for dashed phrases, Mm-Hmm, Uh-Huh, OK etc. those are frequent in AMI
+        # and will be added to dictionary
+        text = re.sub(r"MM HMM", "MM-HMM", text)
+        text = re.sub(r"UH HUH", "UH-HUH", text)
+        text = re.sub(r"(\b)O K(\b)", "\g<1>OK\g<2>", text)
+        return text
+
+
 # IHM and MDM audio requires grouping multiple channels of AudioSource into
 # one Recording.
 
@@ -372,16 +399,32 @@ def prepare_audio_grouped(
     channel_wavs = groupby(lambda p: p.parts[-3], audio_paths)
 
     recordings = []
-    for session_name, channel_paths in channel_wavs.items():
+    for session_name, channel_paths in tqdm(
+        channel_wavs.items(), desc="Processing audio files"
+    ):
         audio_sf = sf.SoundFile(str(channel_paths[0]))
 
+        sources = []
+        all_mono = True
+        for idx, audio_path in enumerate(sorted(channel_paths)):
+            audio = sf.SoundFile(str(audio_path))
+            if audio.channels > 1:
+                logging.warning(
+                    f"Skipping recording {session_name} since it has a stereo channel"
+                )
+                all_mono = False
+                break
+            sources.append(
+                AudioSource(type="file", channels=[idx], source=str(audio_path))
+            )
+
+        if not all_mono:
+            continue
+
         recordings.append(
             Recording(
                 id=session_name,
-                sources=[
-                    AudioSource(type="file", channels=[idx], source=str(audio_path))
-                    for idx, audio_path in enumerate(sorted(channel_paths))
-                ],
+                sources=sources,
                 sampling_rate=audio_sf.samplerate,
                 num_samples=audio_sf.frames,
                 duration=audio_sf.frames / audio_sf.samplerate,
@@ -399,7 +442,7 @@ def prepare_audio_single(
     import soundfile as sf
 
     recordings = []
-    for audio_path in audio_paths:
+    for audio_path in tqdm(audio_paths, desc="Processing audio files"):
         session_name = audio_path.parts[-3]
         audio_sf = sf.SoundFile(str(audio_path))
         recordings.append(
@@ -434,7 +477,7 @@ def prepare_supervision_ihm(
     }
 
     segments = []
-    for recording in audio:
+    for recording in tqdm(audio, desc="Preparing supervisions"):
         # AMI IHM can have multiple audio sources for each recording
         for source in recording.sources:
             # For each source, "channels" will always be a one-element list
@@ -483,20 +526,20 @@ def prepare_supervision_other(
         annotation_by_id[key[0]].extend(value)
 
     segments = []
-    for recording in audio:
+    for recording in tqdm(audio, desc="Preparing supervisions"):
         annotation = annotation_by_id.get(recording.id)
         # In these mic settings, all sources (1 for ihm-mix and sdm and 16 for mdm)
         # will share supervision.
-        source = recording.sources[0]
         if annotation is None:
             logging.warning(f"No annotation found for recording {recording.id}")
             continue
 
-        if len(source.channels) > 1:
+        if any(len(source.channels) > 1 for source in recording.sources):
             logging.warning(
                 f"More than 1 channels in recording {recording.id}. "
-                f"Creating supervision for channel 0 only."
+                f"Skipping this recording."
             )
+            continue
 
         for seg_idx, seg_info in enumerate(annotation):
             duration = seg_info.end_time - seg_info.start_time
@@ -523,7 +566,7 @@ def prepare_ami(
     output_dir: Optional[Pathlike] = None,
     mic: Optional[str] = "ihm",
     partition: Optional[str] = "full-corpus",
-    normalize_text: bool = True,
+    normalize_text: str = "kaldi",
 ) -> Dict[str, Dict[str, Union[RecordingSet, SupervisionSet]]]:
     """
     Returns the manifests which consist of the Recordings and Supervisions
@@ -532,7 +575,7 @@ def prepare_ami(
     :param output_dir: Pathlike, the path where to write the manifests.
     :param mic: str {'ihm','ihm-mix','sdm','mdm'}, type of mic to use.
     :param partition: str {'full-corpus','full-corpus-asr','scenario-only'}, AMI official data split
-    :param normalize_text: bool, whether to normalize text to uppercase
+    :param normalize_text: str {'none', 'upper', 'kaldi'} normalization of text
     :return: a Dict whose key is ('train', 'dev', 'eval'), and the values are dicts of manifests under keys
         'recordings' and 'supervisions'.
 
@@ -562,7 +605,7 @@ def prepare_ami(
                 f"No annotations directory specified and no zip file found in {data_dir}"
             )
     # Prepare annotations which is a list of segment-level transcriptions
-    annotations = parse_ami_annotations(annotations_dir, normalize_text)
+    annotations = parse_ami_annotations(annotations_dir, normalize=normalize_text)
 
     # Audio
     logging.info("Preparing recording manifests")
@@ -603,8 +646,8 @@ def prepare_ami(
 
         # Write to output directory if a path is provided
         if output_dir is not None:
-            audio_part.to_json(output_dir / f"recordings_{part}.json")
-            supervision_part.to_json(output_dir / f"supervisions_{part}.json")
+            audio_part.to_file(output_dir / f"recordings_{part}.jsonl")
+            supervision_part.to_file(output_dir / f"supervisions_{part}.jsonl")
 
         audio_part, supervision_part = fix_manifests(audio_part, supervision_part)
         validate_recordings_and_supervisions(audio_part, supervision_part)
