@@ -89,7 +89,7 @@ def download_cmu_arctic(
     speakers: Sequence[str] = SPEAKERS,
     force_download: Optional[bool] = False,
     base_url: Optional[str] = BASE_URL,
-) -> None:
+) -> Path:
     """
     Download and untar the CMU Arctic dataset.
 
@@ -97,6 +97,7 @@ def download_cmu_arctic(
     :param speakers: a list of speakers to download. By default, downloads all.
     :param force_download: Bool, if True, download the tars no matter if the tars exist.
     :param base_url: str, the url of CMU Arctic download site.
+    :return: the path to downloaded and extracted directory with data.
     """
     target_dir = Path(target_dir)
     target_dir.mkdir(parents=True, exist_ok=True)
@@ -120,6 +121,8 @@ def download_cmu_arctic(
             tar.extractall(path=target_dir)
         completed_detector.touch()
 
+    return target_dir
+
 
 def prepare_cmu_arctic(
     corpus_dir: Pathlike,
