@@ -30,23 +30,24 @@ def download_mobvoihotwords(
     target_dir: Pathlike = ".",
     force_download: Optional[bool] = False,
     base_url: Optional[str] = "http://www.openslr.org/resources",
-) -> None:
+) -> Path:
     """
     Downdload and untar the dataset
 
     :param target_dir: Pathlike, the path of the dir to storage the dataset.
     :param force_download: Bool, if True, download the tars no matter if the tars exist.
     :param base_url: str, the url of the OpenSLR resources.
+    :return: the path to downloaded and extracted directory with data.
     """
 
     url = f"{base_url}/87"
     target_dir = Path(target_dir)
     target_dir.mkdir(parents=True, exist_ok=True)
+    corpus_dir = target_dir / "MobvoiHotwords"
     dataset_tar_name = "mobvoi_hotword_dataset.tgz"
     resources_tar_name = "mobvoi_hotword_dataset_resources.tgz"
     for tar_name in [dataset_tar_name, resources_tar_name]:
         tar_path = target_dir / tar_name
-        corpus_dir = target_dir / "MobvoiHotwords"
         extracted_dir = corpus_dir / tar_name[:-4]
         completed_detector = extracted_dir / ".completed"
         if completed_detector.is_file():
@@ -61,6 +62,8 @@ def download_mobvoihotwords(
             tar.extractall(path=corpus_dir)
         completed_detector.touch()
 
+    return corpus_dir
+
 
 def prepare_mobvoihotwords(
     corpus_dir: Pathlike, output_dir: Optional[Pathlike] = None
