@@ -79,7 +79,7 @@ ISOCODE2LANG = {
 def download_mtedx(
     target_dir: Pathlike = ".",
     languages: Optional[Union[str, Sequence[str]]] = "all",
-) -> None:
+) -> Path:
     """
     Download and untar the dataset.
 
@@ -89,6 +89,7 @@ def download_mtedx(
     :param: languages: A str or sequence of strings specifying which
         languages to download. The default 'all', downloads all available
         languages.
+    :return: the path to downloaded and extracted directory with data.
     """
     target_dir = Path(target_dir) / "mtedx_corpus"
     target_dir.mkdir(parents=True, exist_ok=True)
@@ -115,6 +116,8 @@ def download_mtedx(
             tar.extractall(path=target_dir)
         completed_detector.touch()
 
+    return target_dir
+
 
 ###############################################################################
 #                              Prepare MTEDx
