@@ -35,7 +35,7 @@ def download_musan(
     target_dir: Pathlike = ".",
     url: Optional[str] = MUSAN_URL,
     force_download: Optional[bool] = False,
-) -> None:
+) -> Path:
     """
     Download and untar the MUSAN corpus.
 
@@ -48,15 +48,17 @@ def download_musan(
 
     tar_name = "musan.tar.gz"
     tar_path = target_dir / tar_name
+    corpus_dir = target_dir / "musan"
     completed_detector = target_dir / ".musan_completed"
     if completed_detector.is_file():
         logging.info(f"Skipping {tar_name} because {completed_detector} exists.")
-        return
+        return corpus_dir
     if force_download or not tar_path.is_file():
         urlretrieve_progress(url, filename=tar_path, desc=f"Downloading {tar_name}")
     with tarfile.open(tar_path) as tar:
         tar.extractall(path=target_dir)
         completed_detector.touch()
+    return corpus_dir
 
 
 def prepare_musan(
