@@ -1,4 +1,3 @@
-import gzip
 import itertools
 import json
 import warnings
@@ -7,18 +6,32 @@ from typing import Any, Dict, Generator, Iterable, Optional, Type, Union
 
 import yaml
 
-from lhotse.utils import Pathlike
+from lhotse.utils import Pathlike, is_module_available
+from lhotse.workarounds import gzip_open_robust
 
 # TODO: figure out how to use some sort of typing stubs
 #  so that linters/static checkers don't complain
 Manifest = Any  # Union['RecordingSet', 'SupervisionSet', 'FeatureSet', 'CutSet']
 
 
+def open_best(path: Pathlike, mode: str = "r"):
+    if is_module_available("smart_open"):
+        from smart_open import smart_open
+
+        # This will work with JSONL anywhere that smart_open supports, e.g. cloud storage.
+        open_fn = smart_open
+    else:
+        compressed = str(path).endswith(".gz")
+        if compressed and "t" not in mode and "b" not in mode:
+            # Opening as bytes not requested explicitly, use "t" to tell gzip to handle unicode.
+            mode = mode + "t"
+        open_fn = gzip_open_robust if compressed else open
+
+    return open_fn(path, mode)
+
+
 def save_to_yaml(data: Any, path: Pathlike) -> None:
-    compressed = str(path).endswith(".gz")
-    opener = gzip.open if compressed else open
-    mode = "wt" if compressed else "w"
-    with opener(path, mode) as f:
+    with open_best(path, "w") as f:
         try:
             # When pyyaml is installed with C extensions, it can speed up the (de)serialization noticeably
             yaml.dump(data, stream=f, Dumper=yaml.CSafeDumper)
@@ -27,8 +40,7 @@ def save_to_yaml(data: Any, path: Pathlike) -> None:
 
 
 def load_yaml(path: Pathlike) -> dict:
-    opener = gzip.open if str(path).endswith(".gz") else open
-    with opener(path) as f:
+    with open_best(path) as f:
         try:
             # When pyyaml is installed with C extensions, it can speed up the (de)serialization noticeably
             return yaml.load(stream=f, Loader=yaml.CSafeLoader)
@@ -48,17 +60,13 @@ class YamlMixin:
 
 def save_to_json(data: Any, path: Pathlike) -> None:
     """Save the data to a JSON file. Will use GZip to compress it if the path ends with a ``.gz`` extension."""
-    compressed = str(path).endswith(".gz")
-    opener = gzip.open if compressed else open
-    mode = "wt" if compressed else "w"
-    with opener(path, mode) as f:
+    with open_best(path, "w") as f:
         json.dump(data, f, indent=2)
 
 
 def load_json(path: Pathlike) -> Union[dict, list]:
     """Load a JSON file. Also supports compressed JSON with a ``.gz`` extension."""
-    opener = gzip.open if str(path).endswith(".gz") else open
-    with opener(path) as f:
+    with open_best(path) as f:
         return json.load(f)
 
 
@@ -74,21 +82,17 @@ class JsonMixin:
 
 def save_to_jsonl(data: Iterable[Dict[str, Any]], path: Pathlike) -> None:
     """Save the data to a JSON file. Will use GZip to compress it if the path ends with a ``.gz`` extension."""
-    compressed = str(path).endswith(".gz")
-    opener = gzip.open if compressed else open
-    mode = "wt" if compressed else "w"
-    with opener(path, mode) as f:
+    with open_best(path, "w") as f:
         for item in data:
             print(json.dumps(item), file=f)
 
 
 def load_jsonl(path: Pathlike) -> Generator[Dict[str, Any], None, None]:
     """Load a JSON file. Also supports compressed JSON with a ``.gz`` extension."""
-    opener = gzip.open if str(path).endswith(".gz") else open
-    with opener(path) as f:
+    with open_best(path) as f:
         for line in f:
             # The temporary variable helps fail fast
-            ret = json.loads(line)
+            ret = decode_json_line(line)
             yield ret
 
 
@@ -110,7 +114,7 @@ class SequentialJsonlWriter:
     This writer can be useful for continuing to write files that were previously
     stopped -- it will open the existing file and scan it for item IDs to skip
     writing them later. It can also be queried for existing IDs so that the user
-    code may skip preparing the corresponding manifets.
+    code may skip preparing the corresponding manifests.
 
     Example:
 
@@ -128,23 +132,25 @@ class SequentialJsonlWriter:
     """
 
     def __init__(self, path: Pathlike, overwrite: bool = True) -> None:
-        self.path = Path(path)
-        assert extension_contains(".jsonl", self.path)
-        self.compressed = extension_contains(".gz", self.path)
-        self._open = gzip.open if self.compressed else open
-        self.mode = "wt" if self.compressed else "w"
+        self.path = path
+        if not extension_contains(".jsonl", self.path):
+            raise InvalidPathExtension(
+                f"SequentialJsonlWriter supports only JSONL format (one JSON item per line), "
+                f"but path='{path}'."
+            )
+        self.mode = "w"
         self.ignore_ids = set()
-        if self.path.is_file() and not overwrite:
-            self.mode = "at" if self.compressed else "a"
-            with self._open(self.path) as f:
+        if Path(self.path).is_file() and not overwrite:
+            self.mode = "a"
+            with open_best(self.path) as f:
                 self.ignore_ids = {
                     data["id"]
-                    for data in (json.loads(line) for line in f)
+                    for data in (decode_json_line(line) for line in f)
                     if "id" in data
                 }
 
     def __enter__(self) -> "SequentialJsonlWriter":
-        self.file = self._open(self.path, self.mode)
+        self.file = open_best(self.path, self.mode)
         return self
 
     def __exit__(self, *args, **kwargs) -> None:
@@ -163,10 +169,14 @@ class SequentialJsonlWriter:
     def contains(self, item: Union[str, Any]) -> bool:
         return item in self
 
-    def write(self, manifest) -> None:
+    def write(self, manifest: Any, flush: bool = False) -> None:
         """
         Serializes a manifest item (e.g. :class:`~lhotse.audio.Recording`,
         :class:`~lhotse.cut.Cut`, etc.) to JSON and stores it in a JSONL file.
+
+        :param manifest: the manifest to be written.
+        :param flush: should we flush the file after writing (ensures the changes
+            are synced with the disk and not just buffered for later writing).
         """
         try:
             if manifest.id in self.ignore_ids:
@@ -174,6 +184,63 @@ class SequentialJsonlWriter:
         except AttributeError:
             pass
         print(json.dumps(manifest.to_dict()), file=self.file)
+        if flush:
+            self.file.flush()
+
+    def open_manifest(self) -> Optional[Manifest]:
+        """
+        Opens the manifest that this writer has been writing to.
+        The manifest is opened in a lazy mode.
+        Returns ``None`` when the manifest is empty.
+        """
+        if not Path(self.path).exists():
+            return None
+        if not self.file.closed:
+            # If the user hasn't finished writing, make sure the latest
+            # changes are propagated.
+            self.file.flush()
+        return load_manifest_lazy(self.path)
+
+
+class InvalidPathExtension(ValueError):
+    pass
+
+
+class InMemoryWriter:
+    """
+    Mimics :class:`.SequentialJsonlWriter` API but doesn't actually perform any I/O.
+    It is used internally to create manifest sets without writing them to disk.
+    """
+
+    def __init__(self):
+        self.items = []
+        # for compatibility with SequentialJsonlWriter
+        self.ignore_ids = frozenset()
+
+    def __enter__(self) -> "InMemoryWriter":
+        return self
+
+    def __exit__(self, *args, **kwargs) -> None:
+        pass
+
+    def __contains__(self, item) -> bool:
+        return False
+
+    def contains(self, item: Union[str, Any]) -> bool:
+        return item in self
+
+    def write(self, manifest, flush: bool = False) -> None:
+        self.items.append(manifest)
+
+    def open_manifest(self) -> Optional[Manifest]:
+        """
+        Return a manifest set. Resolves the proper manifest set class by itself.
+        Returns ``None`` when the manifest is empty.
+        """
+        if not self.items:
+            return None
+        cls = resolve_manifest_set_class(self.items[0])
+        return cls.from_items(self.items)
 
 
 class JsonlMixin:
@@ -187,14 +254,18 @@ class JsonlMixin:
 
     @classmethod
     def open_writer(
-        cls, path: Pathlike, overwrite: bool = True
-    ) -> SequentialJsonlWriter:
+        cls, path: Union[Pathlike, None], overwrite: bool = True
+    ) -> Union[SequentialJsonlWriter, InMemoryWriter]:
         """
         Open a sequential writer that allows to store the manifests one by one,
         without the necessity of storing the whole manifest set in-memory.
         Supports writing to JSONL format (``.jsonl``), with optional gzip
         compression (``.jsonl.gz``).
 
+        .. note:: when ``path`` is ``None``, we will return a :class:`.InMemoryWriter`
+            instead has the same API but stores the manifests in memory.
+            It is convenient when you want to make disk saving optional.
+
         Example:
 
             >>> from lhotse import RecordingSet
@@ -206,7 +277,7 @@ class JsonlMixin:
         This writer can be useful for continuing to write files that were previously
         stopped -- it will open the existing file and scan it for item IDs to skip
         writing them later. It can also be queried for existing IDs so that the user
-        code may skip preparing the corresponding manifets.
+        code may skip preparing the corresponding manifests.
 
         Example:
 
@@ -222,10 +293,47 @@ class JsonlMixin:
             ...         recording = Recording.from_file(path, recording_id=recording_id)
             ...         writer.write(recording)
         """
+        if path is None:
+            return InMemoryWriter()
         return SequentialJsonlWriter(path, overwrite=overwrite)
 
 
 class LazyMixin:
+    def from_items(self, data: Iterable):
+        """
+        Function to be implemented by every sub-class of this mixin.
+        It's expected to create a sub-class instance out of an iterable of items
+        that are held by the sub-class (e.g., ``CutSet.from_items(iterable_of_cuts)``).
+        """
+        raise NotImplemented
+
+    @property
+    def data(self) -> Union[Dict[str, Any], Iterable[Any]]:
+        """
+        Property to be implemented by every sub-class of this mixin.
+        It can either be a regular Python dict holding manifests for eager mode,
+        or a special iterator class for lazy mode.
+        """
+        raise NotImplemented
+
+    def to_eager(self):
+        """
+        Evaluates all lazy operations on this manifest, if any, and returns a copy
+        that keeps all items in memory.
+        If the manifest was "eager" already, this is a no-op and won't copy anything.
+        """
+        if not self.is_lazy:
+            return self
+        cls = type(self)
+        return cls.from_items(self)
+
+    @property
+    def is_lazy(self) -> bool:
+        """
+        Indicates whether this manifest was opened in lazy (read-on-the-fly) mode or not.
+        """
+        return not isinstance(self.data, (dict, list, tuple))
+
     @classmethod
     def from_jsonl_lazy(cls, path: Pathlike) -> Manifest:
         """
@@ -235,6 +343,8 @@ class LazyMixin:
         .. warning:: Opening the manifest in this way might cause some methods that
             rely on random access to fail.
         """
+        from lhotse.lazy import LazyJsonlIterator
+
         return cls(LazyJsonlIterator(path))
 
 
@@ -248,8 +358,8 @@ def grouper(n, iterable):
         yield chunk
 
 
-def extension_contains(ext: str, path: Path) -> bool:
-    return any(ext == sfx for sfx in path.suffixes)
+def extension_contains(ext: str, path: Pathlike) -> bool:
+    return any(ext == sfx for sfx in Path(path).suffixes)
 
 
 def load_manifest(path: Pathlike, manifest_cls: Optional[Type] = None) -> Manifest:
@@ -257,8 +367,6 @@ def load_manifest(path: Pathlike, manifest_cls: Optional[Type] = None) -> Manife
     from lhotse import CutSet, FeatureSet, RecordingSet, SupervisionSet
 
     # Determine the serialization format and read the raw data.
-    path = Path(path)
-    assert path.is_file(), f"No such path: {path}"
     if extension_contains(".jsonl", path):
         raw_data = load_jsonl(path)
         if manifest_cls is None:
@@ -272,7 +380,7 @@ def load_manifest(path: Pathlike, manifest_cls: Optional[Type] = None) -> Manife
     elif extension_contains(".yaml", path):
         raw_data = load_yaml(path)
     else:
-        raise ValueError(f"Not a valid manifest: {path}")
+        raise ValueError(f"Not a valid manifest (does the path exist?): {path}")
     data_set = None
 
     # The parse the raw data into Lhotse's data structures.
@@ -284,6 +392,8 @@ def load_manifest(path: Pathlike, manifest_cls: Optional[Type] = None) -> Manife
     for manifest_type in candidates:
         try:
             data_set = manifest_type.from_dicts(raw_data)
+            if len(data_set) == 0:
+                raise RuntimeError()
             break
         except Exception:
             pass
@@ -292,8 +402,59 @@ def load_manifest(path: Pathlike, manifest_cls: Optional[Type] = None) -> Manife
     return data_set
 
 
+def load_manifest_lazy(path: Pathlike) -> Optional[Manifest]:
+    """
+    Generic utility for reading an arbitrary manifest from a JSONL file.
+    Returns None when the manifest is empty.
+    """
+    assert extension_contains(".jsonl", path)
+    raw_data = iter(load_jsonl(path))
+    try:
+        first = next(raw_data)
+    except StopIteration:
+        return None  # empty manifest
+    item = deserialize_item(first)
+    cls = resolve_manifest_set_class(item)
+    return cls.from_jsonl_lazy(path)
+
+
+def load_manifest_lazy_or_eager(path: Pathlike) -> Optional[Manifest]:
+    """
+    Generic utility for reading an arbitrary manifest.
+    If possible, opens the manifest lazily, otherwise reads everything into memory.
+    """
+    if extension_contains(".jsonl", path):
+        return load_manifest_lazy(path)
+    else:
+        return load_manifest(path)
+
+
+def resolve_manifest_set_class(item):
+    """Returns the right *Set class for a manifest, e.g. Recording -> RecordingSet."""
+    from lhotse import (
+        Features,
+        FeatureSet,
+        Recording,
+        RecordingSet,
+        SupervisionSegment,
+        SupervisionSet,
+    )
+    from lhotse.cut import Cut, CutSet
+
+    if isinstance(item, Recording):
+        return RecordingSet
+    if isinstance(item, SupervisionSegment):
+        return SupervisionSet
+    if isinstance(item, Cut):
+        return CutSet
+    if isinstance(item, Features):
+        return FeatureSet
+    raise ValueError(
+        f"No corresponding 'Set' class is known for item of type: {type(item)}"
+    )
+
+
 def store_manifest(manifest: Manifest, path: Pathlike) -> None:
-    path = Path(path)
     if extension_contains(".jsonl", path):
         manifest.to_jsonl(path)
     elif extension_contains(".json", path):
@@ -313,67 +474,15 @@ class Serializable(JsonMixin, JsonlMixin, LazyMixin, YamlMixin):
         store_manifest(self, path)
 
 
-class LazyJsonlIterator:
-    """
-    LazyJsonlIterator provides the ability to read Lhotse objects from a
-    JSONL file on-the-fly, without reading its full contents into memory.
-
-    This class is designed to be a partial "drop-in" replacement for ordinary dicts
-    to support lazy loading of RecordingSet, SupervisionSet and CutSet.
-    Since it does not support random access reads, some methods of these classes
-    might not work properly.
-    """
-
-    def __init__(self, path: Pathlike) -> None:
-        self.path = Path(path)
-        assert extension_contains(".jsonl", self.path)
-
-    def _reset(self) -> None:
-        opener = gzip.open if str(self.path).endswith(".gz") else open
-        self._file = opener(self.path)
-
-    def __getstate__(self):
-        """
-        Store the state for pickling -- we'll only store the path, and re-initialize
-        this iterator when unpickled. This is necessary to transfer this object across processes
-        for PyTorch's DataLoader workers.
-        """
-        state = {"path": self.path}
-        return state
-
-    def __setstate__(self, state: Dict):
-        """Restore the state when unpickled -- open the jsonl file again."""
-        self.__dict__.update(state)
-
-    def __iter__(self):
-        self._reset()
-        return self
-
-    def __next__(self):
-        line = next(self._file)
-        data = json.loads(line)
-        item = deserialize_item(data)
-        return item
-
-    def values(self):
-        yield from self
-
-    def keys(self):
-        return (item.id for item in self)
-
-    def items(self):
-        return ((item.id, item) for item in self)
-
-    def __len__(self) -> int:
-        return count_newlines_fast(self.path)
-
-
 def deserialize_item(data: dict) -> Any:
     # Figures out what type of manifest is being decoded with some heuristics
     # and returns a Lhotse manifest object rather than a raw dict.
     from lhotse import MonoCut, Features, Recording, SupervisionSegment
+    from lhotse.array import deserialize_array
     from lhotse.cut import MixedCut
 
+    if "shape" in data or "array" in data:
+        return deserialize_array(data)
     if "sources" in data:
         return Recording.from_dict(data)
     if "num_features" in data:
@@ -395,22 +504,39 @@ def deserialize_item(data: dict) -> Any:
     raise ValueError(f"Unexpected cut type during deserialization: '{cut_type}'")
 
 
-def count_newlines_fast(path: Pathlike):
+def deserialize_custom_field(data: Optional[dict]) -> Optional[dict]:
     """
-    Counts newlines in a file using buffered chunk reads.
-    The fastest possible option in Python according to:
-    https://stackoverflow.com/a/68385697/5285891
-    (This is a slightly modified variant of that answer.)
+    Handles deserialization of manifests inside the custom field dictionary
+    (e.g. from :class:`~lhotse.SupervisionSegment` or :class:`~lhotse.MonoCut`).
+
+    Mutates the input in-place, and returns it.
+
+    :param data: the contents of ``manifest.custom`` field.
+    :return: ``custom`` field dict with deserialized manifests (if any),
+        or None when input is None.
     """
+    if data is None:
+        return None
 
-    def _make_gen(reader):
-        b = reader(2 ** 16)
-        while b:
-            yield b
-            b = reader(2 ** 16)
+    from lhotse.array import deserialize_array
 
-    path = Path(path)
-    opener = gzip.open if str(path).endswith(".gz") else open
-    with opener(path, "rb") as f:
-        count = sum(buf.count(b"\n") for buf in _make_gen(f.read))
-    return count
+    # If any of the values in the input are also dicts,
+    # it indicates that might be a serialized array manifest.
+    # We'll try to deserialize it, and if there is an error,
+    # we'll just leave it as it was.
+    for key, value in data.items():
+        if isinstance(value, dict):
+            try:
+                data[key] = deserialize_array(value)
+            except:
+                pass
+
+    return data
+
+
+if is_module_available("orjson"):
+    import orjson
+
+    decode_json_line = orjson.loads
+else:
+    decode_json_line = json.loads
