@@ -1,5 +1,4 @@
 import logging
-import random
 from collections import defaultdict
 from dataclasses import dataclass
 from itertools import groupby, islice
@@ -11,10 +10,10 @@ from typing import (
     List,
     Mapping,
     Optional,
-    Sequence,
     Union,
 )
 
+from lhotse.lazy import AlgorithmMixin
 from lhotse.serialization import Serializable
 from lhotse.utils import (
     Pathlike,
@@ -28,6 +27,7 @@ from lhotse.utils import (
     index_by_id_and_check,
     overspans,
     perturb_num_samples,
+    split_manifest_lazy,
     split_sequence,
 )
 
@@ -290,6 +290,21 @@ class SupervisionSegment:
             else self.recording_id,
         )
 
+    def reverb_rir(self, affix_id: bool = True) -> "SupervisionSegment":
+        """
+        Return a ``SupervisionSegment`` with modified ids.
+
+        :param affix_id: When true, we will modify the ``id`` and ``recording_id`` fields
+            by affixing it with "_rvb".
+        :return: a modified copy of the current ``SupervisionSegment``.
+        """
+
+        return fastcopy(
+            self,
+            id=f"{self.id}_rvb" if affix_id else self.id,
+            recording_id=f"{self.recording_id}_rvb" if affix_id else self.recording_id,
+        )
+
     def trim(self, end: Seconds, start: Seconds = 0) -> "SupervisionSegment":
         """
         Return an identical ``SupervisionSegment``, but ensure that ``self.start`` is not negative (in which case
@@ -370,19 +385,56 @@ class SupervisionSegment:
 
     @staticmethod
     def from_dict(data: dict) -> "SupervisionSegment":
-        return SupervisionSegment(
-            **{
-                key: (
-                    {k: [AlignmentItem(**x) for x in v] for k, v in value.items()}
-                    if key == "alignment"
-                    else value
-                )
-                for key, value in data.items()
+        from lhotse.serialization import deserialize_custom_field
+
+        if "custom" in data:
+            deserialize_custom_field(data["custom"])
+
+        if "alignment" in data:
+            data["alignment"] = {
+                k: [AlignmentItem(**x) for x in v] for k, v in data["alignment"].items()
             }
-        )
+
+        return SupervisionSegment(**data)
+
+    def __setattr__(self, key: str, value: Any):
+        """
+        This magic function is called when the user tries to set an attribute.
+        We use it as syntactic sugar to store custom attributes in ``self.custom``
+        field, so that they can be (de)serialized later.
+        """
+        if key in self.__dataclass_fields__:
+            super().__setattr__(key, value)
+        else:
+            custom = ifnone(self.custom, {})
+            custom[key] = value
+            self.custom = custom
+
+    def __getattr__(self, name: str) -> Any:
+        """
+        This magic function is called when the user tries to access an attribute
+        of :class:`.SupervisionSegment` that doesn't exist.
+        It is used as syntactic sugar for accessing the custom supervision attributes.
+
+        We use it to look up the ``custom`` field: when it's None or empty,
+        we'll just raise AttributeError as usual.
+        If ``item`` is found in ``custom``, we'll return ``self.custom[item]``.
+
+        Example of adding custom metadata and retrieving it as an attribute::
+
+            >>> sup = SupervisionSegment('utt1', recording_id='rec1', start=0,
+            ...                          duration=1, channel=0, text='Yummy.')
+            >>> sup.gps_coordinates = "34.1021097,-79.1553182"
+            >>> coordinates = sup.gps_coordinates
+
+        """
+        try:
+            return self.custom[name]
+        except:
+            raise AttributeError(f"No such attribute: {name}")
 
 
-class SupervisionSet(Serializable, Sequence[SupervisionSegment]):
+class SupervisionSet(Serializable, AlgorithmMixin):
     """
     :class:`~lhotse.supervision.SupervisionSet` represents a collection of segments containing some
     supervision information (see :class:`~lhotse.supervision.SupervisionSegment`),
@@ -435,13 +487,11 @@ class SupervisionSet(Serializable, Sequence[SupervisionSegment]):
         return self.segments == other.segments
 
     @property
-    def is_lazy(self) -> bool:
-        """
-        Indicates whether this manifest was opened in lazy (read-on-the-fly) mode or not.
-        """
-        from lhotse.serialization import LazyJsonlIterator
-
-        return isinstance(self.segments, LazyJsonlIterator)
+    def data(
+        self,
+    ) -> Union[Dict[str, SupervisionSegment], Iterable[SupervisionSegment]]:
+        """Alias property for ``self.segments``"""
+        return self.segments
 
     @property
     def ids(self) -> Iterable[str]:
@@ -451,12 +501,78 @@ class SupervisionSet(Serializable, Sequence[SupervisionSegment]):
     def from_segments(segments: Iterable[SupervisionSegment]) -> "SupervisionSet":
         return SupervisionSet(segments=index_by_id_and_check(segments))
 
+    from_items = from_segments
+
     @staticmethod
     def from_dicts(data: Iterable[Dict]) -> "SupervisionSet":
         return SupervisionSet.from_segments(
             SupervisionSegment.from_dict(s) for s in data
         )
 
+    @staticmethod
+    def from_rttm(path: Union[Pathlike, Iterable[Pathlike]]) -> "SupervisionSet":
+        """
+        Read an RTTM file located at ``path`` (or an iterator) and create a :class:`.SupervisionSet` manifest for them.
+        Can be used to create supervisions from custom RTTM files (see, for example, :class:`lhotse.dataset.DiarizationDataset`).
+
+        .. code:: python
+
+            >>> from lhotse import SupervisionSet
+            >>> sup1 = SupervisionSet.from_rttm('/path/to/rttm_file')
+            >>> sup2 = SupervisionSet.from_rttm(Path('/path/to/rttm_dir').rglob('ref_*'))
+
+        The following description is taken from the [dscore](https://github.com/nryant/dscore#rttm) toolkit:
+
+        Rich Transcription Time Marked (RTTM) files are space-delimited text files
+        containing one turn per line, each line containing ten fields:
+
+        - ``Type``  --  segment type; should always by ``SPEAKER``
+        - ``File ID``  --  file name; basename of the recording minus extension (e.g.,
+        ``rec1_a``)
+        - ``Channel ID``  --  channel (1-indexed) that turn is on; should always be
+        ``1``
+        - ``Turn Onset``  --  onset of turn in seconds from beginning of recording
+        - ``Turn Duration``  -- duration of turn in seconds
+        - ``Orthography Field`` --  should always by ``<NA>``
+        - ``Speaker Type``  --  should always be ``<NA>``
+        - ``Speaker Name``  --  name of speaker of turn; should be unique within scope
+        of each file
+        - ``Confidence Score``  --  system confidence (probability) that information
+        is correct; should always be ``<NA>``
+        - ``Signal Lookahead Time``  --  should always be ``<NA>``
+
+        For instance:
+
+            SPEAKER CMU_20020319-1400_d01_NONE 1 130.430000 2.350 <NA> <NA> juliet <NA> <NA>
+            SPEAKER CMU_20020319-1400_d01_NONE 1 157.610000 3.060 <NA> <NA> tbc <NA> <NA>
+            SPEAKER CMU_20020319-1400_d01_NONE 1 130.490000 0.450 <NA> <NA> chek <NA> <NA>
+
+        :param path: Path to RTTM file or an iterator of paths to RTTM files.
+        :return: a new ``SupervisionSet`` instance containing segments from the RTTM file.
+        """
+        from pathlib import Path
+
+        path = [path] if isinstance(path, (Path, str)) else path
+
+        segments = []
+        for file in path:
+            with open(file, "r") as f:
+                for idx, line in enumerate(f):
+                    parts = line.strip().split()
+                    assert len(parts) == 10, f"Invalid RTTM line in file {file}: {line}"
+                    recording_id = parts[1]
+                    segments.append(
+                        SupervisionSegment(
+                            id=f"{recording_id}-{idx:06d}",
+                            recording_id=recording_id,
+                            channel=int(parts[2]),
+                            start=float(parts[3]),
+                            duration=float(parts[4]),
+                            speaker=parts[7],
+                        )
+                    )
+        return SupervisionSet.from_segments(segments)
+
     def with_alignment_from_ctm(
         self, ctm_file: Pathlike, type: str = "word", match_channel: bool = False
     ) -> "SupervisionSet":
@@ -519,19 +635,6 @@ class SupervisionSet(Serializable, Sequence[SupervisionSegment]):
     def to_dicts(self) -> Iterable[dict]:
         return (s.to_dict() for s in self)
 
-    def shuffle(self, rng: Optional[random.Random] = None) -> "SupervisionSet":
-        """
-        Shuffle the supervision IDs in the current :class:`.SupervisionSet` and return a shuffled copy of self.
-
-        :param rng: an optional instance of ``random.Random`` for precise control of randomness.
-        :return: a shuffled copy of self.
-        """
-        if rng is None:
-            rng = random
-        ids = list(self.ids)
-        rng.shuffle(ids)
-        return SupervisionSet(segments={sid: self[sid] for sid in ids})
-
     def split(
         self, num_splits: int, shuffle: bool = False, drop_last: bool = False
     ) -> List["SupervisionSet"]:
@@ -553,6 +656,27 @@ class SupervisionSet(Serializable, Sequence[SupervisionSegment]):
             )
         ]
 
+    def split_lazy(
+        self, output_dir: Pathlike, chunk_size: int
+    ) -> List["SupervisionSet"]:
+        """
+        Splits a manifest (either lazily or eagerly opened) into chunks, each
+        with ``chunk_size`` items (except for the last one, typically).
+
+        In order to be memory efficient, this implementation saves each chunk
+        to disk in a ``.jsonl.gz`` format as the input manifest is sampled.
+
+        .. note:: For lowest memory usage, use ``load_manifest_lazy`` to open the
+            input manifest for this method.
+
+        :param it: any iterable of Lhotse manifests.
+        :param output_dir: directory where the split manifests are saved.
+            Each manifest is saved at: ``{output_dir}/{split_idx}.jsonl.gz``
+        :param chunk_size: the number of items in each chunk.
+        :return: a list of lazily opened chunk manifests.
+        """
+        return split_manifest_lazy(self, output_dir=output_dir, chunk_size=chunk_size)
+
     def subset(
         self, first: Optional[int] = None, last: Optional[int] = None
     ) -> "SupervisionSet":
@@ -570,13 +694,12 @@ class SupervisionSet(Serializable, Sequence[SupervisionSegment]):
 
         if first is not None:
             assert first > 0
-            if first > len(self):
+            out = SupervisionSet.from_items(islice(self, first))
+            if len(out) < first:
                 logging.warning(
-                    f"SupervisionSet has only {len(self)} items but first {first} required; "
-                    f"not doing anything."
+                    f"SupervisionSet has only {len(out)} items but first {first} were requested."
                 )
-                return self
-            return SupervisionSet.from_segments(islice(self, first))
+            return out
 
         if last is not None:
             assert last > 0
@@ -590,28 +713,6 @@ class SupervisionSet(Serializable, Sequence[SupervisionSegment]):
                 islice(self, len(self) - last, len(self))
             )
 
-    def filter(
-        self, predicate: Callable[[SupervisionSegment], bool]
-    ) -> "SupervisionSet":
-        """
-        Return a new SupervisionSet with the SupervisionSegments that satisfy the `predicate`.
-
-        :param predicate: a function that takes a supervision as an argument and returns bool.
-        :return: a filtered SupervisionSet.
-        """
-        return SupervisionSet.from_segments(seg for seg in self if predicate(seg))
-
-    def map(
-        self, transform_fn: Callable[[SupervisionSegment], SupervisionSegment]
-    ) -> "SupervisionSet":
-        """
-        Map a ``transform_fn`` to the SupervisionSegments and return a new ``SupervisionSet``.
-
-        :param transform_fn: a function that modifies a supervision as an argument.
-        :return: a new ``SupervisionSet`` with modified segments.
-        """
-        return SupervisionSet.from_segments(s.map(transform_fn) for s in self)
-
     def transform_text(self, transform_fn: Callable[[str], str]) -> "SupervisionSet":
         """
         Return a copy of the current ``SupervisionSet`` with the segments having a transformed ``text`` field.
@@ -711,6 +812,3 @@ class SupervisionSet(Serializable, Sequence[SupervisionSegment]):
 
     def __len__(self) -> int:
         return len(self.segments)
-
-    def __add__(self, other: "SupervisionSet") -> "SupervisionSet":
-        return SupervisionSet(segments={**self.segments, **other.segments})
