@@ -1,8 +1,10 @@
 import functools
+import inspect
 import logging
 import math
 import random
 import uuid
+import warnings
 from contextlib import AbstractContextManager, contextmanager
 from dataclasses import asdict, dataclass
 from decimal import Decimal, ROUND_HALF_DOWN, ROUND_HALF_UP
@@ -12,6 +14,7 @@ from typing import (
     Any,
     Callable,
     Dict,
+    Generator,
     Iterable,
     List,
     Optional,
@@ -35,6 +38,7 @@ Decibels = float
 INT16MAX = 32768
 EPSILON = 1e-10
 LOG_EPSILON = math.log(EPSILON)
+DEFAULT_PADDING_VALUE = 0  # used for custom attrs
 
 # This is a utility that generates uuid4's and is set when the user calls
 # the ``fix_random_seed`` function.
@@ -263,6 +267,50 @@ def fastcopy(dataclass_obj: T, **kwargs) -> T:
     return type(dataclass_obj)(**{**dataclass_obj.__dict__, **kwargs})
 
 
+def split_manifest_lazy(
+    it: Iterable[Any], output_dir: Pathlike, chunk_size: int
+) -> List:
+    """
+    Splits a manifest (either lazily or eagerly opened) into chunks, each
+    with ``chunk_size`` items (except for the last one, typically).
+
+    In order to be memory efficient, this implementation saves each chunk
+    to disk in a ``.jsonl.gz`` format as the input manifest is sampled.
+
+    .. note:: For lowest memory usage, use ``load_manifest_lazy`` to open the
+        input manifest for this method.
+
+    :param it: any iterable of Lhotse manifests.
+    :param output_dir: directory where the split manifests are saved.
+        Each manifest is saved at: ``{output_dir}/{split_idx}.jsonl.gz``
+    :param chunk_size: the number of items in each chunk.
+    :return: a list of lazily opened chunk manifests.
+    """
+    from lhotse.serialization import SequentialJsonlWriter
+
+    output_dir = Path(output_dir)
+    output_dir.mkdir(parents=True, exist_ok=True)
+
+    items = iter(it)
+    split_idx = 0
+    splits = []
+    while True:
+        try:
+            written = 0
+            with SequentialJsonlWriter(output_dir / f"{split_idx}.jsonl.gz") as writer:
+                while written < chunk_size:
+                    item = next(items)
+                    writer.write(item)
+                    written += 1
+            split_idx += 1
+        except StopIteration:
+            break
+        finally:
+            splits.append(writer.open_manifest())
+
+    return splits
+
+
 def split_sequence(
     seq: Sequence[Any], num_splits: int, shuffle: bool = False, drop_last: bool = False
 ) -> List[List[Any]]:
@@ -329,6 +377,34 @@ def compute_num_frames(
     return num_frames
 
 
+def compute_num_windows(sig_len: Seconds, win_len: Seconds, hop: Seconds) -> int:
+    """
+    Return a number of windows obtained from signal of length equal to ``sig_len``
+    with windows of ``win_len`` and ``hop`` denoting shift between windows.
+    Examples:
+    ```
+      (sig_len,win_len,hop) -> num_windows # list of windows times
+      (1, 6.1, 3) -> 1  # 0-1
+      (3, 1, 6.1) -> 1  # 0-1
+      (3, 6.1, 1) -> 1  # 0-3
+      (5.9, 1, 3) -> 2  # 0-1, 3-4
+      (5.9, 3, 1) -> 4  # 0-3, 1-4, 2-5, 3-5.9
+      (6.1, 1, 3) -> 3  # 0-1, 3-4, 6-6.1
+      (6.1, 3, 1) -> 5  # 0-3, 1-4, 2-5, 3-6, 4-6.1
+      (5.9, 3, 3) -> 2  # 0-3, 3-5.9
+      (6.1, 3, 3) -> 3  # 0-3, 3-6, 6-6.1
+      (0.0, 3, 3) -> 0
+    ```
+    :param sig_len: Signal length in seconds.
+    :param win_len: Window length in seconds
+    :param hop: Shift between windows in seconds.
+    :return: Number of windows in signal.
+    """
+    n = ceil(max(sig_len - win_len, 0) / hop)
+    b = (sig_len - n * hop) > 0
+    return (sig_len > 0) * (n + int(b))
+
+
 def during_docs_build() -> bool:
     import os
 
@@ -591,3 +667,117 @@ def rich_exception_info(fn):
 
 class NonPositiveEnergyError(ValueError):
     pass
+
+
+# Helper functions to mark a function as deprecated. The following is taken from:
+# https://gist.github.com/kgriffs/8202106
+class DeprecatedWarning(UserWarning):
+    pass
+
+
+def deprecated(message):
+    """Flags a method as deprecated.
+    Args:
+        message: A human-friendly string of instructions, such
+            as: 'Please migrate to add_proxy() ASAP.'
+    """
+
+    def decorator(func):
+        """This is a decorator which can be used to mark functions
+        as deprecated. It will result in a warning being emitted
+        when the function is used."""
+
+        @functools.wraps(func)
+        def wrapper(*args, **kwargs):
+
+            frame = inspect.currentframe().f_back
+
+            warnings.warn_explicit(
+                message,
+                category=DeprecatedWarning,
+                filename=inspect.getfile(frame.f_code),
+                lineno=frame.f_lineno,
+            )
+
+            return func(*args, **kwargs)
+
+        return wrapper
+
+    return decorator
+
+
+class suppress_and_warn:
+    """Context manager to suppress specified exceptions that logs the error message.
+
+    After the exception is suppressed, execution proceeds with the next
+    statement following the with statement.
+
+         >>> with suppress_and_warn(FileNotFoundError):
+         ...     os.remove(somefile)
+         >>> # Execution still resumes here if the file was already removed
+    """
+
+    def __init__(self, *exceptions, enabled: bool = True):
+        self._enabled = enabled
+        self._exceptions = exceptions
+
+    def __enter__(self):
+        pass
+
+    def __exit__(self, exctype, excinst, exctb):
+        if not self._enabled:
+            return
+        # Returning True from __exit__ in a context manager tells Python
+        # to suppress an exception.
+        should_suppress = exctype is not None and issubclass(exctype, self._exceptions)
+        if should_suppress:
+            logging.warning(
+                f"[Suppressed {exctype.__qualname__}] Error message: {excinst}"
+            )
+        return should_suppress
+
+
+def streaming_shuffle(
+    data: Iterable[T],
+    bufsize: int = 10000,
+    rng: Optional[random.Random] = None,
+) -> Generator[T, None, None]:
+    """
+    Shuffle the data in the stream.
+
+    This uses a buffer of size ``bufsize``. Shuffling at
+    startup is less random; this is traded off against
+    yielding samples quickly.
+
+    This code is mostly borrowed from WebDataset; note that we use much larger default
+    buffer size because Cuts are very lightweight and fast to read.
+    https://github.com/webdataset/webdataset/blob/master/webdataset/iterators.py#L145
+
+    .. warning: The order of the elements is expected to be much less random than
+        if the whole sequence was shuffled before-hand with standard methods like
+        ``random.shuffle``.
+
+    :param data: iterator
+    :param bufsize: buffer size for shuffling
+    :param rng: either random module or random.Random instance
+    :return: a generator of cuts, shuffled on-the-fly.
+    """
+    if rng is None:
+        rng = random
+    buf = []
+    startup = True
+    for sample in data:
+        if len(buf) < bufsize:
+            try:
+                buf.append(next(data))
+            except StopIteration:
+                pass
+        k = rng.randint(0, len(buf) - 1)
+        sample, buf[k] = buf[k], sample
+        if startup and len(buf) < bufsize:
+            buf.append(sample)
+            continue
+        startup = False
+        yield sample
+    for sample in buf:
+        yield sample
